from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "السبت",
        "1": "الأحد",
        "2": "الإثنين",
        "3": "الثلاثاء",
        "4": "الأربعاء",
        "5": "الخميس",
        "6": "الجمعة",
    }

    MONTH_NAMES = {
        "01": "كانون الثّاني",
        "02": "شباط",
        "03": "آذار",
        "04": "نيسان",
        "05": "أيّار",
        "06": "حزيران",
        "07": "تمّوز",
        "08": "آب",
        "09": "أيلول",
        "10": "تشرين الأول",
        "11": "تشرين الثاني",
        "12": "كانون الأول",
    }

    centuries = [
        "الأول",
        "الثاني",
        "الثالث",
        "الرابع",
        "الخامس",
        "السادس",
        "السابع",
        "الثامن",
        "التاسع",
        "العاشر",
        "الحادي عشر",
        "الثاني عشر",
        "الثالث عشر",
        "الرابع عشر",
        "الخامس عشر",
        "السادس عشر",
        "الثامن عشر",
        "التاسع عشر",
        "العشرين",
        "الحادي والعشرين",
        "الثاني والعشرين",
    ]

    countries = [
        {
            "timezones": ["أوروب/أندورا"],
            "alpha-2-code": "AD",
            "continent": "أوروبا",
            "name": "أندورا",
            "capital": "أندورا لا فيلا",
        },
        {
            "timezones": ["آسيا/كابل"],
            "alpha-2-code": "AF",
            "continent": "آسيا",
            "name": "أفغانستان",
            "capital": "كابل",
        },
        {
            "timezones": ["أمريكا/أنتيغوا"],
            "alpha-2-code": "AG",
            "continent": "أمريكا الشمالية",
            "name": "أنتيغوا وباربودا",
            "capital": "سانت جونز",
        },
        {
            "timezones": ["أوروبا/تيرانا"],
            "alpha-2-code": "AL",
            "continent": "أوروبا",
            "name": "ألبانيا",
            "capital": "تيرانا",
        },
        {
            "timezones": ["آسيا/يريفان"],
            "alpha-2-code": "AM",
            "continent": "آسيا",
            "name": "أرمينيا",
            "capital": "يريفان",
        },
        {
            "timezones": ["إفريقيا/لواندا"],
            "alpha-2-code": "AO",
            "continent": "إفريقيا",
            "name": "أنغولا",
            "capital": "لواندا",
        },
        {
            "timezones": [
                "أمريكا/الأرجنتين/بوينس_آيرس",
                "أمريكا/الأرجنتين/Cordoba",
                "أمريكا/الأرجنتين/خوخوي",
                "أمريكا/الأرجنتين/توكومان",
                "أمريكا/الأرجنتين/كاتاماركا",
                "أمريكا/الأرجنتين/لا_ريوخا",
                "أمريكا/الأرجنتين/سان_خوان",
                "أمريكا/الأرجنتين/مندوزا",
                "أمريكا/الأرجنتين/ريو_غاليغوس",
                "أمريكا/الأرجنتين/أوشوايا",
            ],
            "alpha-2-code": "AR",
            "continent": "أمريكا الجنوبية",
            "name": "الأرجنتين",
            "capital": "بوينس آيرس",
        },
        {
            "timezones": ["أوروبا/النمسا"],
            "alpha-2-code": "AT",
            "continent": "أوروبا",
            "name": "النمسا",
            "capital": "فيينا",
        },
        {
            "timezones": [
                "أستراليا/لورد_هاو",
                "أستراليا/هوبارت",
                "أستراليا/كري",
                "أستراليا/ملبورن",
                "أستراليا/سدني",
                "أستراليا/بروكن_هل",
                "أستراليا/بريزبن",
                "أستراليا/ليندمان",
                "أستراليا/أديلايد",
                "أستراليا/داروين",
                "أستراليا/برث",
            ],
            "alpha-2-code": "AU",
            "continent": "أوقيانوسيا",
            "name": "أستراليا",
            "capital": "كانبرا",
        },
        {
            "timezones": ["آسيا/باكو"],
            "alpha-2-code": "AZ",
            "continent": "آسيا",
            "name": "أذربيجان",
            "capital": "باكو",
        },
        {
            "timezones": ["أمريكا/باربادوس"],
            "alpha-2-code": "BB",
            "continent": "أمريكا الشمالية",
            "name": "باربادوس",
            "capital": "بريدج تاون",
        },
        {
            "timezones": ["آسيا/دكا"],
            "alpha-2-code": "BD",
            "continent": "آسيا",
            "name": "بنغلادش",
            "capital": "دكا",
        },
        {
            "timezones": ["أوروبا/بروكسل"],
            "alpha-2-code": "BE",
            "continent": "أوروبا",
            "name": "بلجيكا",
            "capital": "بروكسل",
        },
        {
            "timezones": ["إفريقيا/واغادوغو"],
            "alpha-2-code": "BF",
            "continent": "إفريقيا",
            "name": "بوركينا فاسو",
            "capital": "واغادوغو",
        },
        {
            "timezones": ["أوروبا/صوفيا"],
            "alpha-2-code": "BG",
            "continent": "أوروبا",
            "name": "بلغاريا",
            "capital": "صوفيا",
        },
        {
            "timezones": ["آسيا/البحرين"],
            "alpha-2-code": "BH",
            "continent": "آسيا",
            "name": "البحرين",
            "capital": "المنامة",
        },
        {
            "timezones": ["إفريقيا/بوجمبورا"],
            "alpha-2-code": "BI",
            "continent": "إفريقيا",
            "name": "بوروندي",
            "capital": "بوجمبورا",
        },
        {
            "timezones": ["إفريقيا/بورتو نوفو"],
            "alpha-2-code": "BJ",
            "continent": "إفريقيا",
            "name": "بنين",
            "capital": "بورتو نوفو",
        },
        {
            "timezones": ["آسيا/بروناي"],
            "alpha-2-code": "BN",
            "continent": "آسيا",
            "name": "اتحاد بروناي (دار السلام)",
            "capital": "بندر سري بكاوان",
        },
        {
            "timezones": ["أمريكا/لاباز"],
            "alpha-2-code": "BO",
            "continent": "أمريكا الجنوبية",
            "name": "بوليفيا",
            "capital": "سوكري",
        },
        {
            "timezones": [
                "أمريكا/نورونها",
                "أمريكا/بليم",
                "أمريكا/فورتاليزا",
                "أمريكا/ريسيفي",
                "أمريكا/أراغوينا",
                "أمريكا/ماسايو",
                "أمريكا/باهيا",
                "أمريكا/ساو_باولو",
                "أمريكا/كامبو_غراندي",
                "أمريكا/كويابا",
                "أمريكا/بورتو_فاليو",
                "أمريكا/بوا_فيستا",
                "أمريكا/ماناوس",
                "أمريكا/إيرونيبي",
                "أمريكا/ريو_برانكو",
            ],
            "alpha-2-code": "BR",
            "continent": "أمريكا الجنوبية",
            "name": "البرازيل",
            "capital": "برازيليا",
        },
        {
            "timezones": ["أمريكا/ناساو"],
            "alpha-2-code": "BS",
            "continent": "أمريكا الشمالية",
            "name": "باهاماس",
            "capital": "ناساو",
        },
        {
            "timezones": ["آسيا/تيمفو"],
            "alpha-2-code": "BT",
            "continent": "آسيا",
            "name": "بوتان",
            "capital": "تيمفو",
        },
        {
            "timezones": ["إفريقيا/غابورون"],
            "alpha-2-code": "BW",
            "continent": "إفريقيا",
            "name": "بوتسوانا",
            "capital": "غابورون",
        },
        {
            "timezones": ["أوروبا/مينسك"],
            "alpha-2-code": "BY",
            "continent": "أوروبا",
            "name": "روسيا البيضاء",
            "capital": "مينسك",
        },
        {
            "timezones": ["أمريكا/بليز"],
            "alpha-2-code": "BZ",
            "continent": "أمريكا الشمالية",
            "name": "بليز",
            "capital": "بلموبان",
        },
        {
            "timezones": [
                "أمريكا/سينت_جونز",
                "أمريكا/هاليفاكس",
                "أمريكا/جليس_باي",
                "أمريكا/مونكتون",
                "أمريكا/جووس_باي",
                "أمريكا/بلانك_سابلون",
                "أمريكا/مونتريال",
                "أمريكا/تورونتو",
                "أمريكا/نيبيغون",
                "أمريكا/ثاندر_باي",
                "أمريكا/بانغيرتانغ",
                "أمريكا/إيكواليوت",
                "أمريكا/أتيكوكان",
                "أمريكا/رانكن_إنلت",
                "أمريكا/وينيبيغ",
                "أمريكا/رايني_ريفر",
                "أمريكا/كامبريدج_باي",
                "أمريكا/ريجينا",
                "أمريكا/سويفت_كارنت",
                "أمريكا/إدمونتون",
                "أمريكا/يلو_نايف",
                "أمريكا/إنوفك",
                "أمريكا/دوسن_كريك",
                "أمريكا/فانكوفر",
                "أمريكا/وايت_هورس",
                "أمريكا/داوسون",
            ],
            "alpha-2-code": "CA",
            "continent": "أمريكا الشمالية",
            "name": "كندا",
            "capital": "أوتاوا",
        },
        {
            "timezones": ["إفريقيا/كينشاسا", "إفريقيا/لوبومباشي"],
            "alpha-2-code": "CD",
            "continent": "إفريقيا",
            "name": "جمهورية الكونغو الديمقراطية",
            "capital": "كينشاسا",
        },
        {
            "timezones": ["إفريقيا/برازافيل"],
            "alpha-2-code": "CG",
            "continent": "إفريقيا",
            "name": "جمهورية الكونغو",
            "capital": "برازافيل",
        },
        {
            "timezones": ["إفريقيا/أبيدجان"],
            "alpha-2-code": "CI",
            "continent": "إفريقيا",
            "name": "ساحل العاج",
            "capital": "ياموسوكرو",
        },
        {
            "timezones": ["أمريكا/سانتياغو", "المحيط_الهاديء/جزيرة_القيامة"],
            "alpha-2-code": "CL",
            "continent": "أمريكا الجنوبية",
            "name": "تشيلي",
            "capital": "سانتياغو",
        },
        {
            "timezones": ["إفريقيا/دوالا"],
            "alpha-2-code": "CM",
            "continent": "إفريقيا",
            "name": "الكاميرون",
            "capital": "ياوندي",
        },
        {
            "timezones": [
                "آسيا/شانغهاي",
                "آسيا/هاربن",
                "آسيا/تشونغتشينغ",
                "آسيا/أورومتشي",
                "آسيا/كاشغر",
            ],
            "alpha-2-code": "CN",
            "continent": "آسيا",
            "name": "جمهورية الصين الشعبية",
            "capital": "بكين",
        },
        {
            "timezones": ["أمريكا/بوغوتا"],
            "alpha-2-code": "CO",
            "continent": "أمريكا الجنوبية",
            "name": "كولومبيا",
            "capital": "بوغوتا",
        },
        {
            "timezones": ["أمريكا/كوستاريكا"],
            "alpha-2-code": "CR",
            "continent": "أمريكا الشمالية",
            "name": "كوستاريكا",
            "capital": "سان خوسيه",
        },
        {
            "timezones": ["أمريكا/هافانا"],
            "alpha-2-code": "CU",
            "continent": "أمريكا الشمالية",
            "name": "كوبا",
            "capital": "هافانا",
        },
        {
            "timezones": ["الأطلنطي/الرأس_الأخضر"],
            "alpha-2-code": "CV",
            "continent": "إفريقيا",
            "name": "جمهورية الرأس الأخضر",
            "capital": "برايا",
        },
        {
            "timezones": ["آسيا/نيقوسيا"],
            "alpha-2-code": "CY",
            "continent": "آسيا",
            "name": "قبرص",
            "capital": "نيقوسيا",
        },
        {
            "timezones": ["أوروبا/براغ"],
            "alpha-2-code": "CZ",
            "continent": "أوروبا",
            "name": "جمهورية التشيك",
            "capital": "براغ",
        },
        {
            "timezones": ["أوروبا/برلين"],
            "alpha-2-code": "DE",
            "continent": "أوروبا",
            "name": "ألمانيا",
            "capital": "برلين",
        },
        {
            "timezones": ["إفريقيا/جيبوتي"],
            "alpha-2-code": "DJ",
            "continent": "إفريقيا",
            "name": "جيبوتي",
            "capital": "جيبوتي",
        },
        {
            "timezones": ["أوروبا/كوبنهاغن"],
            "alpha-2-code": "DK",
            "continent": "أوروبا",
            "name": "الدنمارك",
            "capital": "كوبنهاغن",
        },
        {
            "timezones": ["أمريكا/دومينيكا"],
            "alpha-2-code": "DM",
            "continent": "أمريكا الشمالية",
            "name": "دومينيكا",
            "capital": "روسياو",
        },
        {
            "timezones": ["أمريكا/سانتو_دومينغو"],
            "alpha-2-code": "DO",
            "continent": "أمريكا الشمالية",
            "name": "جمهورية الدومينيكان",
            "capital": "سانتو دومينغو",
        },
        {
            "timezones": ["أمريكا/غواياكيل", "المحيط_الهاديء/أرخبيل_غالاباغوس"],
            "alpha-2-code": "EC",
            "continent": "أمريكا الجنوبية",
            "name": "الإكوادور",
            "capital": "كيتو",
        },
        {
            "timezones": ["أوروبا/تالين"],
            "alpha-2-code": "EE",
            "continent": "أوروبا",
            "name": "إستونيا",
            "capital": "تالين",
        },
        {
            "timezones": ["إفريقيا/القاهرة"],
            "alpha-2-code": "EG",
            "continent": "إفريقيا",
            "name": "مصر",
            "capital": "القاهرة",
        },
        {
            "timezones": ["إفريقيا/أسمرة"],
            "alpha-2-code": "ER",
            "continent": "إفريقيا",
            "name": "إرتيريا",
            "capital": "أسمرة",
        },
        {
            "timezones": ["إفريقيا/أديس أبابا"],
            "alpha-2-code": "ET",
            "continent": "إفريقيا",
            "name": "إثيوبيا",
            "capital": "أديس أبابا",
        },
        {
            "timezones": ["أوروبا/هلسنكي"],
            "alpha-2-code": "FI",
            "continent": "أوروبا",
            "name": "فنلندا",
            "capital": "هلسنكي",
        },
        {
            "timezones": ["المحيط_الهاديء/فيجي"],
            "alpha-2-code": "FJ",
            "continent": "أوقيانوسيا",
            "name": "فيجي",
            "capital": "سوفا",
        },
        {
            "timezones": ["أوروبا/باريس"],
            "alpha-2-code": "FR",
            "continent": "أوروبا",
            "name": "فرنسا",
            "capital": "باريس",
        },
        {
            "timezones": ["إفريقيا/ليبرفيل"],
            "alpha-2-code": "GA",
            "continent": "إفريقيا",
            "name": "الغابون",
            "capital": "ليبرفيل",
        },
        {
            "timezones": ["آسيا/تبليسي"],
            "alpha-2-code": "GE",
            "continent": "آسيا",
            "name": "جورجيا",
            "capital": "تبليسي",
        },
        {
            "timezones": ["إفريقيا/أكرا"],
            "alpha-2-code": "GH",
            "continent": "إفريقيا",
            "name": "غانا",
            "capital": "أكرا",
        },
        {
            "timezones": ["إفريقيا/بانجول"],
            "alpha-2-code": "GM",
            "continent": "إفريقيا",
            "name": "غامبيا",
            "capital": "بانجول",
        },
        {
            "timezones": ["إفريقيا/كوناكري"],
            "alpha-2-code": "GN",
            "continent": "إفريقيا",
            "name": "غينيا",
            "capital": "كوناكري",
        },
        {
            "timezones": ["أوروبا/أثينا"],
            "alpha-2-code": "GR",
            "continent": "أوروبا",
            "name": "اليونان",
            "capital": "أثينا",
        },
        {
            "timezones": ["أمريكا/غواتيمالا"],
            "alpha-2-code": "GT",
            "continent": "أمريكا الشمالية",
            "name": "غواتيمالا",
            "capital": "غواتيمالا سيتي",
        },
        {
            "timezones": ["أمريكا/غواتيمالا"],
            "alpha-2-code": "GT",
            "continent": "أمريكا الشمالية",
            "name": "هايتي",
            "capital": "بورت أو برانس",
        },
        {
            "timezones": ["إفريقيا/بيساو"],
            "alpha-2-code": "GW",
            "continent": "إفريقيا",
            "name": "غينيا بيساو",
            "capital": "بيساو",
        },
        {
            "timezones": ["أمريكا/غيانا"],
            "alpha-2-code": "GY",
            "continent": "أمريكا الجنوبية",
            "name": "غيانا",
            "capital": "جورج تاون",
        },
        {
            "timezones": ["أمريكا/تيجوسيجالبا"],
            "alpha-2-code": "HN",
            "continent": "أمريكا الشمالية",
            "name": "هندوراس",
            "capital": "تيجوسيجالبا",
        },
        {
            "timezones": ["أوروبا/بودابست"],
            "alpha-2-code": "HU",
            "continent": "أوروبا",
            "name": "هنغاريا",
            "capital": "بودابست",
        },
        {
            "timezones": [
                "آسيا/جاكرتا",
                "آسيا/بونتياناك",
                "آسيا/ماكاسار",
                "آسيا/جايابورا",
            ],
            "alpha-2-code": "ID",
            "continent": "آسيا",
            "name": "إندونسيا",
            "capital": "جاكرتا",
        },
        {
            "timezones": ["أوروبا/دبلن"],
            "alpha-2-code": "IE",
            "continent": "أوروبا",
            "name": "إيرلندا",
            "capital": "دبلن",
        },
        {
            "timezones": ["آسيا/القدس"],
            "alpha-2-code": "IL",
            "continent": "آسيا",
            "name": "فلسطين",
            "capital": "القدس",
        },
        {
            "timezones": ["آسيا/كالكتا"],
            "alpha-2-code": "IN",
            "continent": "آسيا",
            "name": "الهند",
            "capital": "نيو دلهي",
        },
        {
            "timezones": ["آسيا/بغداد"],
            "alpha-2-code": "IQ",
            "continent": "آسيا",
            "name": "العراق",
            "capital": "بغداد",
        },
        {
            "timezones": ["آسيا/طهران"],
            "alpha-2-code": "IR",
            "continent": "آسيا",
            "name": "إيران",
            "capital": "طهران",
        },
        {
            "timezones": ["الأطلنطي/ريكيافيك"],
            "alpha-2-code": "IS",
            "continent": "أوروبا",
            "name": "آيسلندا",
            "capital": "ريكيافيك",
        },
        {
            "timezones": ["أوروبا/روما"],
            "alpha-2-code": "IT",
            "continent": "أوروبا",
            "name": "إيطاليا",
            "capital": "روما",
        },
        {
            "timezones": ["أمريكا/جامايكا"],
            "alpha-2-code": "JM",
            "continent": "أمريكا الشمالية",
            "name": "جامايكا",
            "capital": "كينغستون",
        },
        {
            "timezones": ["آسيا/عمّان"],
            "alpha-2-code": "JO",
            "continent": "آسيا",
            "name": "الأردن",
            "capital": "عمّان",
        },
        {
            "timezones": ["آسيا/طوكيو"],
            "alpha-2-code": "JP",
            "continent": "آسيا",
            "name": "اليابان",
            "capital": "طوكيو",
        },
        {
            "timezones": ["إفريقيا/نيروبي"],
            "alpha-2-code": "KE",
            "continent": "إفريقيا",
            "name": "كينيا",
            "capital": "نيروبي",
        },
        {
            "timezones": ["آسيا/بشكيك"],
            "alpha-2-code": "KG",
            "continent": "آسيا",
            "name": "قيرغيزستان",
            "capital": "بشكيك",
        },
        {
            "timezones": [
                "المحيط_الهاديء/تاراوا",
                "المحيط_الهاديء/إيديربيري",
                "المحيط_الهاديء/كريتيماتي",
            ],
            "alpha-2-code": "KI",
            "continent": "أوقيانوسيا",
            "name": "كيريباتي",
            "capital": "جنوب تاراوا",
        },
        {
            "timezones": ["آسيا/بيونغ_يانغ"],
            "alpha-2-code": "KP",
            "continent": "آسيا",
            "name": "كوريا الشمالية",
            "capital": "بيونغ يانغ",
        },
        {
            "timezones": ["آسيا/سيؤول"],
            "alpha-2-code": "KR",
            "continent": "آسيا",
            "name": "؛كوريا الجنوبية",
            "capital": "سيؤول",
        },
        {
            "timezones": ["آسيا/الكويت"],
            "alpha-2-code": "KW",
            "continent": "آسيا",
            "name": "الكويت",
            "capital": "الكويت",
        },
        {
            "timezones": ["آسيا/بيروت"],
            "alpha-2-code": "LB",
            "continent": "آسيا",
            "name": "لبنان",
            "capital": "بيروت",
        },
        {
            "timezones": ["أوروبا/فادوز"],
            "alpha-2-code": "LI",
            "continent": "أوروبا",
            "name": "ليختنشتاين",
            "capital": "فادوز",
        },
        {
            "timezones": ["إفريقيا/مونروفيا"],
            "alpha-2-code": "LR",
            "continent": "إفريقيا",
            "name": "ليبيريا",
            "capital": "مونروفيا",
        },
        {
            "timezones": ["إفريقيا/ماسيرو"],
            "alpha-2-code": "LS",
            "continent": "إفريقيا",
            "name": "ليسوتو",
            "capital": "ماسيرو",
        },
        {
            "timezones": ["أوروبا/فيلنيوس"],
            "alpha-2-code": "LT",
            "continent": "أوروبا",
            "name": "ليتوانيا",
            "capital": "فيلنيوس",
        },
        {
            "timezones": ["أوروبا/لوكسمبرغ"],
            "alpha-2-code": "LU",
            "continent": "أوروبا",
            "name": "لوكسمبرغ",
            "capital": "لوكسمبرغ سيتي",
        },
        {
            "timezones": ["أوروبا/ربيغ"],
            "alpha-2-code": "LV",
            "continent": "أوروبا",
            "name": "لاتفيا",
            "capital": "ربيغ",
        },
        {
            "timezones": ["إفريقيا/طرابلس"],
            "alpha-2-code": "LY",
            "continent": "إفريقيا",
            "name": "ليبيا",
            "capital": "طرابلس",
        },
        {
            "timezones": ["الهندي/أنتاناناريفو"],
            "alpha-2-code": "MG",
            "continent": "إفريقيا",
            "name": "مدغشقر",
            "capital": "أنتاناناريفو",
        },
        {
            "timezones": ["المحيط_الهاديء/ماجورو", "المحيط_الهاديء/كواجلين_أتول"],
            "alpha-2-code": "MH",
            "continent": "أوقيانوسيا",
            "name": "جزر مارشال",
            "capital": "ماجورو",
        },
        {
            "timezones": ["أوروبا/سكوبيه"],
            "alpha-2-code": "MK",
            "continent": "أوروبا",
            "name": "جمهورية مقدونيا",
            "capital": "سكوبيه",
        },
        {
            "timezones": ["إفريقيا/باماكو"],
            "alpha-2-code": "ML",
            "continent": "إفريقيا",
            "name": "مالي",
            "capital": "باماكو",
        },
        {
            "timezones": ["آسيا/رانغون"],
            "alpha-2-code": "MM",
            "continent": "آسيا",
            "name": "ميانمار",
            "capital": "نايبيداو",
        },
        {
            "timezones": ["آسيا/أولان_باتور", "آسيا/Hovd", "آسيا/تشويبالسان"],
            "alpha-2-code": "MN",
            "continent": "آسيا",
            "name": "مانغوليا",
            "capital": "أولان باتور",
        },
        {
            "timezones": ["إفريقيا/نواكشط"],
            "alpha-2-code": "MR",
            "continent": "إفريقيا",
            "name": "موريتانيا",
            "capital": "نواكشط",
        },
        {
            "timezones": ["أوروبا/مالطا"],
            "alpha-2-code": "MT",
            "continent": "أوروبا",
            "name": "مالطا",
            "capital": "فاليتا",
        },
        {
            "timezones": ["الهندي/موريشيوس"],
            "alpha-2-code": "MU",
            "continent": "إفريقيا",
            "name": "موريشيوس",
            "capital": "بور لويس",
        },
        {
            "timezones": ["الهندي/جزر_المالديف"],
            "alpha-2-code": "MV",
            "continent": "آسيا",
            "name": "جمهورية المالديف",
            "capital": "ماليه",
        },
        {
            "timezones": ["إفريقيا/بلانتاير"],
            "alpha-2-code": "MW",
            "continent": "إفريقيا",
            "name": "ملاوي",
            "capital": "ليلونغوي",
        },
        {
            "timezones": [
                "أمريكا/ميكسيكو_سيتي",
                "أمريكا/كانكون",
                "أمريكا/ميرديا",
                "أمريكا/مونتيري",
                "أمريكا/مازاتلان",
                "أمريكا/شيواوا",
                "أمريكا/ارموسييو_سونورا",
                "أمريكا/تيخوانا",
            ],
            "alpha-2-code": "MX",
            "continent": "أمريكا الشمالية",
            "name": "المكسيك",
            "capital": "ميكسيكو سيتي§",
        },
        {
            "timezones": ["آسيا/كوالا_لامبور", "آسيا/Kuching"],
            "alpha-2-code": "MY",
            "continent": "آسيا",
            "name": "ماليزيا",
            "capital": "كوالا لامبور",
        },
        {
            "timezones": ["إفريقيا/مابوتو"],
            "alpha-2-code": "MZ",
            "continent": "إفريقيا",
            "name": "موزمبيق",
            "capital": "مابوتو",
        },
        {
            "timezones": ["إفريقيا/ويندهوك"],
            "alpha-2-code": "NA",
            "continent": "إفريقيا",
            "name": "ناميبيا",
            "capital": "ويندهوك",
        },
        {
            "timezones": ["إفريقيا/نيامي"],
            "alpha-2-code": "NE",
            "continent": "إفريقيا",
            "name": "النيجر",
            "capital": "نيامي",
        },
        {
            "timezones": ["إفريقيا/لاغوس"],
            "alpha-2-code": "NG",
            "continent": "إفريقيا",
            "name": "نيجيريا",
            "capital": "أبوجا",
        },
        {
            "timezones": ["أمريكا/ماناغوا"],
            "alpha-2-code": "NI",
            "continent": "أمريكا الشمالية",
            "name": "نيكاراغوا",
            "capital": "ماناغوا",
        },
        {
            "timezones": ["أوروبا/أمستردام"],
            "alpha-2-code": "NL",
            "continent": "أوروبا",
            "name": "هولندا",
            "capital": "أمستردام",
        },
        {
            "timezones": ["أوروبا/أوسلو"],
            "alpha-2-code": "NO",
            "continent": "أوروبا",
            "name": "النرويج",
            "capital": "أوسلو",
        },
        {
            "timezones": ["آسيا/كاتماندو"],
            "alpha-2-code": "NP",
            "continent": "آسيا",
            "name": "النيبال",
            "capital": "كاتماندو",
        },
        {
            "timezones": ["المحيط_الهاديء/ناورو"],
            "alpha-2-code": "NR",
            "continent": "أوقيانوسيا",
            "name": "ناورو",
            "capital": "يارين",
        },
        {
            "timezones": ["المحيط_الهاديء/أوكلاند", "المحيط_الهاديء/تشاتهام"],
            "alpha-2-code": "NZ",
            "continent": "أوقيانوسيا",
            "name": "نيوزيلاندا",
            "capital": "ويلينغتون",
        },
        {
            "timezones": ["آسيا/مسقط"],
            "alpha-2-code": "OM",
            "continent": "آسيا",
            "name": "عمان",
            "capital": "مسقط",
        },
        {
            "timezones": ["أمريكا/بنما"],
            "alpha-2-code": "PA",
            "continent": "أمريكا الشمالية",
            "name": "بنما",
            "capital": "بنما",
        },
        {
            "timezones": ["أمريكا/ليما"],
            "alpha-2-code": "PE",
            "continent": "أمريكا الجنوبية",
            "name": "البيرو",
            "capital": "ليما",
        },
        {
            "timezones": ["المحيط_الهاديء/بورت_مورسبي"],
            "alpha-2-code": "PG",
            "continent": "أوقيانوسيا",
            "name": "بابوا غينيا الجديدة",
            "capital": "بورت مورسبي",
        },
        {
            "timezones": ["آسيا/مانيلا"],
            "alpha-2-code": "PH",
            "continent": "آسيا",
            "name": "الفيليبين",
            "capital": "مانيلا",
        },
        {
            "timezones": ["آسيا/كاراتشي"],
            "alpha-2-code": "PK",
            "continent": "آسيا",
            "name": "باكستان",
            "capital": "إسلام أباد",
        },
        {
            "timezones": ["أوروبا/وارسو"],
            "alpha-2-code": "PL",
            "continent": "أوروبا",
            "name": "بولندا",
            "capital": "وارسو",
        },
        {
            "timezones": ["أوروبا/لشبونة", "الأطلنطي/ماديرا", "الأطلنطي/الأزور"],
            "alpha-2-code": "PT",
            "continent": "أوروبا",
            "name": "البرتغال",
            "capital": "لشبونة",
        },
        {
            "timezones": ["المحيط_الهاديء/بالاو"],
            "alpha-2-code": "PW",
            "continent": "أوقيانوسيا",
            "name": "بالاو",
            "capital": "نجيرولمد",
        },
        {
            "timezones": ["أمريكا/أسونسيون"],
            "alpha-2-code": "PY",
            "continent": "أمريكا الجنوبية",
            "name": "بابرغوي",
            "capital": "أسونسيون",
        },
        {
            "timezones": ["آسيا/قطر"],
            "alpha-2-code": "QA",
            "continent": "آسيا",
            "name": "قطر",
            "capital": "الدوحة",
        },
        {
            "timezones": ["أوروبا/بوخارست"],
            "alpha-2-code": "RO",
            "continent": "أوروبا",
            "name": "رومانيا",
            "capital": "بوخارست",
        },
        {
            "timezones": [
                "أوروبا/كالينينغراد",
                "أوروبا/موسكو",
                "أوروبا/Volgograd",
                "أوروبا/سمارة",
                "آسيا/يكاترينبورغ",
                "آسيا/أومسك",
                "آسيا/نوفوسيبيرسك",
                "آسيا/كراسنوياسك",
                "آسيا/إروتسك",
                "آسيا/ياكوتسك",
                "آسيا/فالديفوستوك",
                "آسيا/ساخالن",
                "آسيا/ماغادان",
                "آسيا/كامشتكا",
                "آسيا/أنادير",
            ],
            "alpha-2-code": "RU",
            "continent": "أوروبا",
            "name": "روسيا",
            "capital": "موسكو",
        },
        {
            "timezones": ["إفريقيا/كيغالي"],
            "alpha-2-code": "RW",
            "continent": "إفريقيا",
            "name": "رواندا",
            "capital": "كيغالي",
        },
        {
            "timezones": ["آسيا/الرياض"],
            "alpha-2-code": "SA",
            "continent": "آسيا",
            "name": "المملكة العربية السعودية",
            "capital": "الرياض",
        },
        {
            "timezones": ["المحيط_الهاديء/غوادالكانال"],
            "alpha-2-code": "SB",
            "continent": "أوقيانوسيا",
            "name": "جزر سولمون",
            "capital": "هونيارا",
        },
        {
            "timezones": ["الهندي/ماهي"],
            "alpha-2-code": "SC",
            "continent": "إفريقيا",
            "name": "سيشل",
            "capital": "فيكتوريا",
        },
        {
            "timezones": ["إفريقيا/الخرطوم"],
            "alpha-2-code": "SD",
            "continent": "إفريقيا",
            "name": "السودان",
            "capital": "الخرطوم",
        },
        {
            "timezones": ["أوروبا/ستوكهولم"],
            "alpha-2-code": "SE",
            "continent": "أوروبا",
            "name": "السويد",
            "capital": "ستوكهولم",
        },
        {
            "timezones": ["آسيا/سنغافورة"],
            "alpha-2-code": "SG",
            "continent": "آسيا",
            "name": "سنغافورة",
            "capital": "سنغافورة",
        },
        {
            "timezones": ["أوروبا/ليوبليانا"],
            "alpha-2-code": "SI",
            "continent": "أوروبا",
            "name": "سلوفانيا",
            "capital": "ليوبليانا",
        },
        {
            "timezones": ["أوروبا/براتيسلافا"],
            "alpha-2-code": "SK",
            "continent": "أوروبا",
            "name": "سلوفاكيا",
            "capital": "براتيسلافا",
        },
        {
            "timezones": ["إفريقيا/فريتاون"],
            "alpha-2-code": "SL",
            "continent": "إفريقيا",
            "name": "سيراليون",
            "capital": "فريتاون",
        },
        {
            "timezones": ["أوروبا/سان_مارينو"],
            "alpha-2-code": "SM",
            "continent": "أوروبا",
            "name": "جمهورية سان مارينو",
            "capital": "سان مارينو",
        },
        {
            "timezones": ["إفريقيا/داكار"],
            "alpha-2-code": "SN",
            "continent": "إفريقيا",
            "name": "السنغال",
            "capital": "داكار",
        },
        {
            "timezones": ["إفريقيا/مقديشو"],
            "alpha-2-code": "SO",
            "continent": "إفريقيا",
            "name": "الصومال",
            "capital": "مقديشو",
        },
        {
            "timezones": ["أمريكا/باراماريبو"],
            "alpha-2-code": "SR",
            "continent": "أمريكا الجنوبية",
            "name": "Suriname",
            "capital": "باراماريبو",
        },
        {
            "timezones": ["إفريقيا/ساو_تومي"],
            "alpha-2-code": "ST",
            "continent": "إفريقيا",
            "name": " ساو تومي وبرينسيب",
            "capital": "ساو تومي",
        },
        {
            "timezones": ["آسيا/دممشق"],
            "alpha-2-code": "SY",
            "continent": "آسيا",
            "name": "سوريا",
            "capital": "دمشق",
        },
        {
            "timezones": ["إفريقيا/لومي"],
            "alpha-2-code": "TG",
            "continent": "إفريقيا",
            "name": "توغو",
            "capital": "لومي",
        },
        {
            "timezones": ["آسيا/بانغوك"],
            "alpha-2-code": "TH",
            "continent": "آسيا",
            "name": "تايلند",
            "capital": "بناغوك",
        },
        {
            "timezones": ["آسيا/دوشنبه"],
            "alpha-2-code": "TJ",
            "continent": "آسيا",
            "name": "طاجكيستان",
            "capital": "دوشنبه",
        },
        {
            "timezones": ["آسيا/عشق_آباد"],
            "alpha-2-code": "TM",
            "continent": "آسيا",
            "name": "تركمانستان",
            "capital": "عشق آباد",
        },
        {
            "timezones": ["إفريقيا/تونس"],
            "alpha-2-code": "TN",
            "continent": "إفريقيا",
            "name": "تونس",
            "capital": "تونس",
        },
        {
            "timezones": ["المحيط_الهاديء/تونغاتابو"],
            "alpha-2-code": "TO",
            "continent": "أوقيانوسيا",
            "name": "تونغا",
            "capital": "نوكو ألوفا",
        },
        {
            "timezones": ["أوروبا/إسطنبول"],
            "alpha-2-code": "TR",
            "continent": "آسيا",
            "name": "تركيا",
            "capital": "أنقرة",
        },
        {
            "timezones": ["أمريكا/بورت_أوف_سبين"],
            "alpha-2-code": "TT",
            "continent": "أمريكا الشمالية",
            "name": "ترينيداد وتوباغو",
            "capital": "بورت أوف سبين",
        },
        {
            "timezones": ["المحيط_الهاديء/فونافوتي"],
            "alpha-2-code": "TV",
            "continent": "أوقيانوسيا",
            "name": "توفالو",
            "capital": "فونافوتي",
        },
        {
            "timezones": ["إفريقيا/دار_السلام"],
            "alpha-2-code": "TZ",
            "continent": "إفريقيا",
            "name": "تانزانيا",
            "capital": "دودوما",
        },
        {
            "timezones": [
                "أوروبا/كييف",
                "أوروبا/أوجهورود",
                "أوروبا/زاباروجيا",
                "أوروبا/سيمفروبول",
            ],
            "alpha-2-code": "UA",
            "continent": "أوروبا",
            "name": "أوكرانيا",
            "capital": "كييف",
        },
        {
            "timezones": ["إفريقيا/كامبالا"],
            "alpha-2-code": "UG",
            "continent": "إفريقيا",
            "name": "أوغندا",
            "capital": "كامبالا",
        },
        {
            "timezones": [
                "أمريكا/نيويورك",
                "أمريكا/ديترويت",
                "أمريكا/كنتاكي/لويسفيل",
                "أمريكا/كنتاكي/مونتيسللو",
                "أمريكا/إنديانا/إنديانابولس",
                "أمريكا/إنديانا/مارنغو",
                "أمريكا/إنديانا/نوكس",
                "أمريكا/إنديانا/فيفاي",
                "أمريكا/شيكاغو",
                "أمريكا/إنديانا/فانسان",
                "أمريكا/إنديانا/بيترزبيرغ",
                "أمريكا/مينومني",
                "أمريكا/نورث_داكوتا/سينتر",
                "أمريكا/نورث_داكوتا/نيو_سالم",
                "أمريكا/دنفر",
                "أمريكا/بويسي",
                "أمريكا/شيبروك",
                "أمريكا/فينيكس",
                "أمريكا/لوس_أنجيلوس",
                "أمريكا/أنكوريج",
                "أمريكا/جونو",
                "أمريكا/ياكوتات",
                "أمريكا/نوم",
                "أمريكا/أداك",
                "المحيط_الهاديء/هونولولو",
            ],
            "alpha-2-code": "US",
            "continent": "أمريكا الشمالية",
            "name": "الولايات المتحدة الأمريكية",
            "capital": "واشنطن",
        },
        {
            "timezones": ["أمريكا/مونتفيدو"],
            "alpha-2-code": "UY",
            "continent": "أمريكا الجنوبية",
            "name": "أوروغواي",
            "capital": "مونتفيدو",
        },
        {
            "timezones": ["آسيا/سمرقند", "آسيا/طشقند"],
            "alpha-2-code": "UZ",
            "continent": "آسيا",
            "name": "أوزبكستان",
            "capital": "طشقند",
        },
        {
            "timezones": ["أوروبا/الفاتيكان"],
            "alpha-2-code": "VA",
            "continent": "أوروبا",
            "name": "الفاتيكان",
            "capital": "الفاتيكان",
        },
        {
            "timezones": ["أمريكا/كاركاس"],
            "alpha-2-code": "VE",
            "continent": "أمريكا الجنوبية",
            "name": "فنزويلا",
            "capital": "كاركاس",
        },
        {
            "timezones": ["آسيا/سايغون"],
            "alpha-2-code": "VN",
            "continent": "آسيا",
            "name": "فيتنام",
            "capital": "هانوي",
        },
        {
            "timezones": ["المحيط_الهاديء/أيفاتي"],
            "alpha-2-code": "VU",
            "continent": "أوقيانوسيا",
            "name": "فانواتو",
            "capital": "بورت فيلا",
        },
        {
            "timezones": ["آسيا/عدن"],
            "alpha-2-code": "YE",
            "continent": "آسيا",
            "name": "اليمن",
            "capital": "صنعاء",
        },
        {
            "timezones": ["إفريقيا/لوساكا"],
            "alpha-2-code": "ZM",
            "continent": "إفريقيا",
            "name": "زامبيا",
            "capital": "لوساكا",
        },
        {
            "timezones": ["إفريقيا/هراري"],
            "alpha-2-code": "ZW",
            "continent": "إفريقيا",
            "name": "زيمبابوي",
            "capital": "هراري",
        },
        {
            "timezones": ["إفريقيا/الجزائر"],
            "alpha-2-code": "DZ",
            "continent": "إفريقيا",
            "name": "الجزائر",
            "capital": "الجزائر",
        },
        {
            "timezones": ["أوروبا/سراييفو"],
            "alpha-2-code": "BA",
            "continent": "أوروبا",
            "name": "البوسنة والهرسك",
            "capital": "سراييفو",
        },
        {
            "timezones": ["آسيا/بنوم_بنه"],
            "alpha-2-code": "KH",
            "continent": "آسيا",
            "name": "كمبوديا",
            "capital": "بنوم بنه",
        },
        {
            "timezones": ["إفريقيا/بانغي"],
            "alpha-2-code": "CF",
            "continent": "إفريقيا",
            "name": "جمهورية أفريقيا الوسطى",
            "capital": "بانغي",
        },
        {
            "timezones": ["إفريقيا/نجامينا"],
            "alpha-2-code": "TD",
            "continent": "إفريقيا",
            "name": "تشاد",
            "capital": "نجامينا",
        },
        {
            "timezones": ["الهندي/كومورو"],
            "alpha-2-code": "KM",
            "continent": "إفريقيا",
            "name": "جزر القمر",
            "capital": "موروني",
        },
        {
            "timezones": ["أوروبا/زغرب"],
            "alpha-2-code": "HR",
            "continent": "أوروبا",
            "name": "كرواتيا",
            "capital": "زغرب",
        },
        {
            "timezones": ["آسيا/ديلي"],
            "alpha-2-code": "TL",
            "continent": "آسيا",
            "name": "تيمور الشرقية",
            "capital": "ديلي",
        },
        {
            "timezones": ["أمريكا/السلفادور"],
            "alpha-2-code": "SV",
            "continent": "أمريكا الشمالية",
            "name": "السلفادور",
            "capital": "سان سلفادور",
        },
        {
            "timezones": ["إفريقيا/مالابو"],
            "alpha-2-code": "GQ",
            "continent": "إفريقيا",
            "name": "غينيا الاستوائية",
            "capital": "مالابو",
        },
        {
            "timezones": ["أمريكا/غرينادا"],
            "alpha-2-code": "GD",
            "continent": "أمريكا الشمالية",
            "name": "غرينادا",
            "capital": "سانت جورجز",
        },
        {
            "timezones": [
                "آسيا/ألماتي",
                "آسيا/كيزيلوردا",
                "آسيا/أقتوبي",
                "آسيا/أقتاو",
                "آسيا/أورال",
            ],
            "alpha-2-code": "KZ",
            "continent": "آسيا",
            "name": "كازاخستان",
            "capital": "أستانة",
        },
        {
            "timezones": ["آسيا/فيينتيان"],
            "alpha-2-code": "LA",
            "continent": "آسيا",
            "name": "لاوس",
            "capital": "فيينتيان",
        },
        {
            "timezones": [
                "المحيط_الهاديء/تشوك",
                "المحيط_الهاديء/بونابي",
                "المحيط_الهاديء/كورساي",
            ],
            "alpha-2-code": "FM",
            "continent": "أوقيانوسيا",
            "name": "ولايات ميكرونيسيا المتحدة",
            "capital": "باليكير",
        },
        {
            "timezones": ["أوروبا/كيشيناو"],
            "alpha-2-code": "MD",
            "continent": "أوروبا",
            "name": "مولدافيا",
            "capital": "كيشيناو",
        },
        {
            "timezones": ["أوروبا/موناكو"],
            "alpha-2-code": "MC",
            "continent": "أوروبا",
            "name": "موناكو",
            "capital": "موناكو",
        },
        {
            "timezones": ["أوروبا/بودغوريتسا"],
            "alpha-2-code": "ME",
            "continent": "أوروبا",
            "name": "الجبل الأسود",
            "capital": "بودغوريتسا",
        },
        {
            "timezones": ["إفريقيا/الدار_البيضاء"],
            "alpha-2-code": "MA",
            "continent": "إفريقيا",
            "name": "المغرب",
            "capital": "الرباط",
        },
        {
            "timezones": ["أمريكا/سانت_كيتس"],
            "alpha-2-code": "KN",
            "continent": "أمريكا الشمالية",
            "name": "سانت كيتس ونيفيس",
            "capital": "باستير",
        },
        {
            "timezones": ["أمريكا/سانت_لوسيا"],
            "alpha-2-code": "LC",
            "continent": "أمريكا الشمالية",
            "name": "سانت لوسيا",
            "capital": "كاستريس",
        },
        {
            "timezones": ["أمريكا/سينت_فينسينت"],
            "alpha-2-code": "VC",
            "continent": "أمريكا الشمالية",
            "name": "سانت فينسنت والغرينادين",
            "capital": "كينغستاون",
        },
        {
            "timezones": ["المحيط_الهاديء/أبيا"],
            "alpha-2-code": "WS",
            "continent": "أوقيانوسيا",
            "name": "ساموا",
            "capital": "أبيا",
        },
        {
            "timezones": ["أوروبا/بلغراد"],
            "alpha-2-code": "RS",
            "continent": "أوروبا",
            "name": "صربيا",
            "capital": "بلغراد",
        },
        {
            "timezones": ["إفريقيا/جوهانسبرغ"],
            "alpha-2-code": "ZA",
            "continent": "إفريقيا",
            "name": "جنوب إفريقيا",
            "capital": "بريتوريا",
        },
        {
            "timezones": ["أوروبا/مدريد", "إفريقيا/سبتة", "الأطلنطي/الكناري"],
            "alpha-2-code": "ES",
            "continent": "أوروبا",
            "name": "إسبانيا",
            "capital": "مدريد",
        },
        {
            "timezones": ["آسيا/كولمبو"],
            "alpha-2-code": "LK",
            "continent": "آسيا",
            "name": "سريلانكا",
            "capital": "سري جاياواردنابورا كوتي",
        },
        {
            "timezones": ["إفريقيا/مبابان"],
            "alpha-2-code": "SZ",
            "continent": "إفريقيا",
            "name": "سوازيلاند",
            "capital": "مبابان",
        },
        {
            "timezones": ["أوروبا/زيورخ"],
            "alpha-2-code": "CH",
            "continent": "أوروبا",
            "name": "سويسرا",
            "capital": "برن",
        },
        {
            "timezones": ["آسيا/دبي"],
            "alpha-2-code": "AE",
            "continent": "آسيا",
            "name": "الإمارات العربية المتحدة",
            "capital": "أبو ظبي",
        },
        {
            "timezones": ["أوروبا/لندن"],
            "alpha-2-code": "GB",
            "continent": "أوروبا",
            "name": "المملكة المتحدة",
            "capital": "لندن",
        },
    ]

    AM_PM = {
        "AM": "ص",
        "PM": "م",
    }

    def month_name(self) -> str:
        month = self.date("%m")
        return self.MONTH_NAMES[month]

    def am_pm(self) -> str:
        date = self.date("%p")
        return self.AM_PM[date]

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{last_name}}, {{first_name}}",
    )

    first_names = (
        "Tomas",
        "Lukas",
        "Mantas",
        "Deividas",
        "Arnas",
        "Artūras",
        "Karolis",
        "Dovydas",
        "Dominykas",
        "Darius",
        "Edvinas",
        "Jonas",
        "Martynas",
        "Kajus",
        "Donatas",
        "Andrius",
        "Matas",
        "Rokas",
        "Augustas",
        "Danielius",
        "Mindaugas",
        "Paulius",
        "Marius",
        "Armandas",
        "Edgaras",
        "Jokūbas",
        "Nedas",
        "Tadas",
        "Nerijus",
        "Simonas",
        "Vytautas",
        "Artūras",
        "Robertas",
        "Eimantas",
        "Arijus",
        "Nojus",
        "Egidijus",
        "Aurimas",
        "Emilis",
        "Laurynas",
        "Edvardas",
        "Joris",
        "Pijus",
        "Erikas",
        "Domas",
        "Vilius",
        "Evaldas",
        "Justinas",
        "Aleksandras",
        "Kristupas",
        "Gabrielius",
        "Benas",
        "Gytis",
        "Arminas",
        "Vakris",
        "Tautvydas",
        "Domantas",
        "Justas",
        "Markas",
        "Antanas",
        "Arūnas",
        "Ernestas",
        "Aronas",
        "Vaidas",
        "Ąžuolas",
        "Titas",
        "Giedrius",
        "Ignas",
        "Povilas",
        "Saulius",
        "Julius",
        "Arvydas",
        "Kęstutis",
        "Rytis",
        "Aistis",
        "Gediminas",
        "Algirdas",
        "Naglis",
        "Irmantas",
        "Rolandas",
        "Aivaras",
        "Simas",
        "Faustas",
        "Ramūnas",
        "Šarūnas",
        "Gustas",
        "Tajus",
        "Dainius",
        "Arnoldas",
        "Linas",
        "Rojus",
        "Adomas",
        "Žygimantas",
        "Ričardas",
        "Orestas",
        "Kipras",
        "Juozas",
        "Audrius",
        "Romualdas",
        "Petras",
        "Eleonora",
        "Raminta",
        "Dovilė",
        "Sandra",
        "Dominyka",
        "Ana",
        "Erika",
        "Kristina",
        "Gintarė",
        "Rūta",
        "Edita",
        "Karina",
        "Živilė",
        "Jolanta",
        "Radvilė",
        "Ramunė",
        "Svetlana",
        "Ugnė",
        "Eglė",
        "Viktorija",
        "Justina",
        "Brigita",
        "Rasa",
        "Marija",
        "Giedrė",
        "Iveta",
        "Sonata",
        "Vitalija",
        "Adrija",
        "Goda",
        "Paulina",
        "Kornelija",
        "Liepa",
        "Vakarė",
        "Milda",
        "Meda",
        "Vaida",
        "Izabelė",
        "Jovita",
        "Irma",
        "Žemyna",
        "Leila",
        "Rimantė",
        "Mantė",
        "Rytė",
        "Perla",
        "Greta",
        "Monika",
        "Ieva",
        "Indrė",
        "Ema",
        "Aurelija",
        "Smiltė",
        "Ingrida",
        "Simona",
        "Amelija",
        "Sigita",
        "Olivija",
        "Laurita",
        "Jorūnė",
        "Leticija",
        "Vigilija",
        "Medėja",
        "Laura",
        "Agnė",
        "Evelina",
        "Kotryna",
        "Lėja",
        "Aušra",
        "Neringa",
        "Gerda",
        "Jurgita",
        "Rusnė",
        "Aušrinė",
        "Rita",
        "Elena",
        "Ineta",
        "Ligita",
        "Vasarė",
        "Vėjūnė",
        "Ignė",
        "Gytė",
        "Ariana",
        "Arielė",
        "Vytė",
        "Eidvilė",
        "Karolina",
        "Miglė",
        "Viltė",
        "Jolanta",
        "Enrika",
        "Aurėja",
        "Vanesa",
        "Darija",
        "Reda",
        "Milana",
        "Rugilė",
        "Diana",
    )

    last_names = (
        "Kazlauskas",
        "Jankauskas",
        "Petrauskas",
        "Pocius",
        "Stankevičius",
        "Vsiliauskas",
        "Žukauskas",
        "Butkus",
        "Paulauskas",
        "Urbonas",
        "Kavaliauskas",
        "Sakalauskas",
        "Žukauskas",
        "Akelis",
        "Ambrasas",
        "Kairys",
        "Kalvaitis",
        "Kalvelis",
        "Kalvėnas",
        "Kaupas",
        "Kiška",
        "Gagys",
        "Gailius",
        "Gailys",
        "Gaižauskas",
        "Gaičiūnas",
        "Galdikas",
        "Gintalas",
        "Ginzburgas",
        "Grinius",
        "Gronskis",
        "Nagys",
        "Naujokas",
        "Narušis",
        "Nausėda",
        "Poška",
        "Povilonis",
    )



from .. import Provider as AddressProvider


class Provider(AddressProvider):

    # Building numbers dont go higher than a 1000
    building_number_formats = ("%##", "%#", "%")

    street_name_formats = ("{{street_prefix}} {{street_suffix}}",)

    street_address_formats = ("{{street_name}} {{building_number}}",)

    street_prefixes = (
        "Gade",
        "Allé",
        "Parkere",
        "Baggade",
        "Gyde",
        "Motorvej",
    )

    street_suffixes = (
        "Åbenrå",
        "Adelgade",
        "Lavendelstræde",
        "Nygade",
        "Pilestræde",
        "Rigensgade",
        "Silkegade",
        "Toldbodgade",
        "Blekingegade",
        "Amalievej",
        "Bülowsvej",
        "Godthåbsvej",
        "Howitzvej",
        "Kingosgade",
        "Martinsvej",
        "Rolighedsvej",
        "Schønbergsgade",
        "Åboulevard",
        "Ewaldsgade",
        "Heinesgade",
        "Kapelvej",
    )

    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    # Postcode should be formatted as described in http://www.nr.dk/danmark.html
    postcode_formats = ("%###",)

    city_formats = ("{{city_name}}",)

    cities = (
        "Copenhagen",
        "Aarhus",
        "Odense",
        "Aalborg",
        "Esbjerg",
        "Randers",
        "Kolding",
        "Horsens",
        "Vejle",
        "Roskilde",
        "Herning",
        "Hørsholm",
        "Elsinore",
        "Silkeborg",
        "Næstved",
        "Fredericia",
        "Viborg",
        "Køge",
        "Holstebro",
        "Taastrup",
        "Slagelse",
        "Hillerød",
        "Holbæk",
        "Sønderborg",
        "Svendborg",
        "Hjørring",
        "Frederikshavn",
        "Nørresundby",
        "Ringsted",
        "Haderslev",
        "Stenløse",
        "Skive",
        "Birkerød",
        "Farum",
        "Smørumnedre",
        "Skanderborg",
        "Nyborg",
        "Nykøbing F",
        "Lillerød",
        "Kalundborg",
        "Frederikssund",
        "Aabenraa",
        "Solrød Strand",
        "Ikast",
        "Middelfart",
        "Grenaa",
        "Korsør",
        "Varde",
        "Rønne",
        "Thisted",
    )

    countries = (
        "Afghanistan",
        "Albanien",
        "Algeriet",
        "Amerikanska Samoa",
        "Andorra",
        "Angola",
        "Anguilla",
        "Antarktis",
        "Antigua och Barbuda",
        "Argentina",
        "Armenien",
        "Aruba",
        "Ascension",
        "Australien",
        "Azerbajdzjan",
        "Bahamas",
        "Bahrain",
        "Bangladesh",
        "Barbados",
        "Belarus",
        "Belgien",
        "Belize",
        "Benin",
        "Bermuda",
        "Bhutan",
        "Bolivia",
        "Bosnien och Hercegovina",
        "Botswana",
        "Brasilien",
        "Brittiska Jungfruöarna",
        "Brunei",
        "Bulgarien",
        "Burkina Faso",
        "Burma",
        "Burundi",
        "Caymanöarna",
        "Centralafrikanska republiken",
        "Chile",
        "Colombia",
        "Cooköarna",
        "Costa Rica",
        "Cypern",
        "Danmark",
        "Diego Garcia",
        "Djibouti",
        "Dominica",
        "Dominikanska republiken",
        "Ecuador",
        "Egypten",
        "Ekvatorialguinea",
        "Elfenbenskusten",
        "El Salvador",
        "Eritrea",
        "Estland",
        "Etiopien",
        "England",
        "Falklandsöarna",
        "Fiji",
        "Filippinerna",
        "Finland",
        "Frankrike",
        "Franska Guyana",
        "Franska Polynesien",
        "Färöarna",
        "Förenade Arabemiraten",
        "Gabon",
        "Gambia",
        "Georgien",
        "Ghana",
        "Gibraltar",
        "Grekland",
        "Grenada",
        "Grönland",
        "Guadeloupe",
        "Guatemala",
        "Guinea",
        "Guinea-Bissau",
        "Guyana",
        "Haiti",
        "Honduras",
        "Hongkong",
        "Indien",
        "Indonesien",
        "Irak",
        "Iran",
        "Irland",
        "Island",
        "Israel",
        "Italien",
        "Jamaica",
        "Japan",
        "Jemen",
        "Jordanien",
        "Kambodja",
        "Kamerun",
        "Kanada",
        "Kap Verde",
        "Kazakstan",
        "Kenya",
        "Kina",
        "Kirgizistan",
        "Kiribati",
        "Komorerna",
        "Kongo-Brazzaville",
        "Kongo-Kinshasa",
        "Kosovo",
        "Kroatien",
        "Kuba",
        "Kuwait",
        "Laos",
        "Lesotho",
        "Lettland",
        "Libanon",
        "Liberia",
        "Libyen",
        "Liechtenstein",
        "Litauen",
        "Luxemburg",
        "Macao",
        "Madagaskar",
        "Makedonien",
        "Malawi",
        "Malaysia",
        "Maldiverna",
        "Mali",
        "Malta",
        "Marianerna",
        "Marocko",
        "Marshallöarna",
        "Martinique",
        "Mauretanien",
        "Mauritius",
        "Mayotte",
        "Mexiko",
        "Midwayöarna",
        "Mikronesiens federerade stater",
        "Moçambique",
        "Moldavien",
        "Monaco",
        "Mongoliet",
        "Montenegro",
        "Montserrat",
        "Namibia",
        "Nauru",
        "Nederländerna",
        "Nederländska Antillerna",
        "Nepal",
        "Nicaragua",
        "Niger",
        "Nigeria",
        "Niue",
        "Nordkorea",
        "Nordmarianerna",
        "Norfolkön",
        "Norge",
        "Nya Kaledonien",
        "Nya Zeeland",
        "Oman",
        "Pakistan",
        "Palau",
        "Palestina",
        "Panama",
        "Papua Nya Guinea",
        "Paraguay",
        "Peru",
        "Pitcairnöarna",
        "Polen",
        "Portugal",
        "Qatar",
        "Réunion",
        "Rumänien",
        "Rwanda",
        "Ryssland",
        "Saint Kitts och Nevis",
        "Saint Lucia",
        "Saint-Pierre och Miquelon",
        "Saint Vincent och Grenadinerna",
        "Salomonöarna",
        "Samoa",
        "Sankta Helena",
        "San Marino",
        "São Tomé och Príncipe",
        "Saudiarabien",
        "Schweiz",
        "Senegal",
        "Serbien",
        "Seychellerna",
        "SierraLeone",
        "Singapore",
        "Sint Maarten",
        "Slovakien",
        "Slovenien",
        "Somalia",
        "Spanien",
        "Sri Lanka",
        "Storbritannien",
        "Sudan",
        "Surinam",
        "Sverige",
        "Swaziland",
        "Sydafrika",
        "Sydkorea",
        "Sydsudan",
        "Syrien",
        "Tadzjikistan",
        "Taiwan",
        "Tanzania",
        "Tchad",
        "Thailand",
        "Tjeckien",
        "Togo",
        "Tokelauöarna",
        "Tonga",
        "Trinidad och Tobago",
        "Tunisien",
        "Turkiet",
        "Turkmenistan",
        "Turks-och Caicosöarna",
        "Tuvalu",
        "Tyskland",
        "Uganda",
        "Ukraina",
        "Ungern",
        "Uruguay",
        "USA",
        "Uzbekistan",
        "Vanuatu",
        "Vatikanstaten",
        "Venezuela",
        "Vietnam",
        "Wake",
        "Wallis-och Futunaöarna",
        "Zambia",
        "Zimbabwe",
        "Österrike",
        "Östtimor",
    )

    # Known as regions in Denmark
    states = (
        "Hovedstaden",
        "Midtjylland",
        "Nordjylland",
        "Sjælland",
        "Syddanmark",
    )

    def street_prefix(self) -> str:
        return self.random_element(self.street_prefixes)

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit



import itertools

from datetime import date
from decimal import Decimal
from typing import Dict, List, Optional, Tuple, Union

from ...typing import GenderType
from .. import BaseProvider


class Provider(BaseProvider):
    """
    This provider is a collection of functions to generate personal profiles and identities.

    """

    def simple_profile(self, sex: Optional[GenderType] = None) -> Dict[str, Union[str, date, GenderType]]:
        """
        Generates a basic profile with personal informations
        """
        sex_ = self.random_element(["F", "M"]) if sex is None else sex
        if sex_ == "F":
            name = self.generator.name_female()
        elif sex_ == "M":
            name = self.generator.name_male()
        return {
            "username": self.generator.user_name(),
            "name": name,
            "sex": sex_,
            "address": self.generator.address(),
            "mail": self.generator.free_email(),
            "birthdate": self.generator.date_of_birth(),
        }

    def profile(
        self, fields: Optional[List[str]] = None, sex: Optional[GenderType] = None
    ) -> Dict[str, Union[str, Tuple[Decimal, Decimal], List[str], date]]:
        """
        Generates a complete profile.
        If "fields" is not empty, only the fields in the list will be returned
        """
        if fields is None:
            fields = []

        d = {
            "job": self.generator.job(),
            "company": self.generator.company(),
            "ssn": self.generator.ssn(),
            "residence": self.generator.address(),
            "current_location": (self.generator.latitude(), self.generator.longitude()),
            "blood_group": "".join(self.random_element(list(itertools.product(["A", "B", "AB", "O"], ["+", "-"])))),
            "website": [self.generator.url() for _ in range(1, self.random_int(2, 5))],
        }

        d = dict(d, **self.generator.simple_profile(sex))
        # field selection
        if len(fields) > 0:
            d = {k: v for k, v in d.items() if k in fields}

        return d



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = ["{{company_prefix}}{{company_suffix}}"]

    company_prefixes = [
        "超艺",
        "和泰",
        "九方",
        "鑫博腾飞",
        "戴硕电子",
        "济南亿次元",
        "海创",
        "创联世纪",
        "凌云",
        "泰麒麟",
        "彩虹",
        "兰金电子",
        "晖来计算机",
        "天益",
        "恒聪百汇",
        "菊风公司",
        "惠派国际公司",
        "创汇",
        "思优",
        "时空盒数字",
        "易动力",
        "飞海科技",
        "华泰通安",
        "盟新",
        "商软冠联",
        "图龙信息",
        "易动力",
        "华远软件",
        "创亿",
        "时刻",
        "开发区世创",
        "明腾",
        "良诺",
        "天开",
        "毕博诚",
        "快讯",
        "凌颖信息",
        "黄石金承",
        "恩悌",
        "雨林木风计算机",
        "双敏电子",
        "维旺明",
        "网新恒天",
        "数字100",
        "飞利信",
        "立信电子",
        "联通时科",
        "中建创业",
        "新格林耐特",
        "新宇龙信息",
        "浙大万朋",
        "MBP软件",
        "昂歌信息",
        "万迅电脑",
        "方正科技",
        "联软",
        "七喜",
        "南康",
        "银嘉",
        "巨奥",
        "佳禾",
        "国讯",
        "信诚致远",
        "浦华众城",
        "迪摩",
        "太极",
        "群英",
        "合联电子",
        "同兴万点",
        "襄樊地球村",
        "精芯",
        "艾提科信",
        "昊嘉",
        "鸿睿思博",
        "四通",
        "富罳",
        "商软冠联",
        "诺依曼软件",
        "东方峻景",
        "华成育卓",
        "趋势",
        "维涛",
        "通际名联",
    ]
    company_suffixes = [n + "有限公司" for n in ["科技", "网络", "信息", "传媒"]]

    def company_prefix(self) -> str:
        return self.random_element(self.company_prefixes)



from ..fil_PH import Provider as FilPhProvider


class Provider(FilPhProvider):
    """Implement lorem provider for ``tl_PH`` locale.

    There is no difference from the |FilPhLoremProvider|.

    .. |FilPhLoremProvider| replace::
        :meth:`FilPhLoremProvider <faker.providers.lorem.fil_PH.Provider>`
    """

    pass



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):

    DAY_NAMES = {
        "0": "Sonntag",
        "1": "Montag",
        "2": "Dienstag",
        "3": "Mittwoch",
        "4": "Donnerstag",
        "5": "Freitag",
        "6": "Samstag",
    }

    MONTH_NAMES = {
        "01": "Jänner",
        "02": "Februar",
        "03": "März",
        "04": "April",
        "05": "Mai",
        "06": "Juni",
        "07": "Juli",
        "08": "August",
        "09": "September",
        "10": "Oktober",
        "11": "November",
        "12": "Dezember",
    }

    def day_of_week(self):
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as AddressProvider


class Provider(AddressProvider):

    city_formats = ("{{city_name}}",)

    street_name_formats = ("{{street_name}}",)
    street_address_formats = ("{{street_name}} {{building_number}}",)
    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    building_number_formats = ("%", "%#", "%##")

    street_suffixes_long = ("ulice", "třída", "nábřeží", "náměstí")
    street_suffixes_short = ("ul.", "tř.", "nábř.", "nám.")

    postcode_formats = (
        "1## ##",
        "2## ##",
        "3## ##",
        "4## ##",
        "5## ##",
        "6## ##",
        "7## ##",
    )

    cities = (
        "Abertamy",
        "Adamov",
        "Andělská Hora",
        "Bakov nad Jizerou",
        "Bavorov",
        "Bechyně",
        "Benešov nad Ploučnicí",
        "Benátky nad Jizerou",
        "Bezdružice",
        "Bečov nad Teplou",
        "Blatná",
        "Blovice",
        "Blšany",
        "Bochov",
        "Bohušovice nad Ohří",
        "Bojkovice",
        "Bor",
        "Borohrádek",
        "Borovany",
        "Boží Dar",
        "Brandýs nad Orlicí",
        "Brno",
        "Broumov",
        "Brtnice",
        "Brumov-Bylnice",
        "Brušperk",
        "Budišov nad Budišovkou",
        "Budyně nad Ohří",
        "Bučovice",
        "Buštěhrad",
        "Bystré",
        "Bystřice",
        "Bystřice nad Pernštejnem",
        "Bystřice pod Hostýnem",
        "Bzenec",
        "Bílovec",
        "Bělá nad Radbuzou",
        "Bělá pod Bezdězem",
        "Březnice",
        "Březová",
        "Březová nad Svitavou",
        "Břidličná",
        "Chabařovice",
        "Chlumec",
        "Chlumec nad Cidlinou",
        "Choceň",
        "Chomutov",
        "Chotěboř",
        "Chrast",
        "Chrastava",
        "Chropyně",
        "Chvaletice",
        "Chyše",
        "Chýnov",
        "Chřibská",
        "Cvikov",
        "Dačice",
        "Dašice",
        "Desná",
        "Deštná",
        "Dobrovice",
        "Dobruška",
        "Dobřany",
        "Dobřichovice",
        "Dobříš",
        "Doksy",
        "Dolní Benešov",
        "Dolní Bousov",
        "Dolní Kounice",
        "Dolní Poustevna",
        "Dubá",
        "Dubí",
        "Dubňany",
        "Duchcov",
        "Děčín",
        "Františkovy Lázně",
        "Fryšták",
        "Frýdek-Místek",
        "Frýdlant",
        "Frýdlant nad Ostravicí",
        "Fulnek",
        "Golčův Jeníkov",
        "Habartov",
        "Habry",
        "Hanušovice",
        "Harrachov",
        "Hartmanice",
        "Havířov",
        "Hejnice",
        "Heřmanův Městec",
        "Hlinsko",
        "Hluboká nad Vltavou",
        "Hluk",
        "Hodkovice nad Mohelkou",
        "Holice",
        "Holýšov",
        "Hora Svaté Kateřiny",
        "Horažďovice",
        "Horní Benešov",
        "Horní Blatná",
        "Horní Bříza",
        "Horní Cerekev",
        "Horní Jelení",
        "Horní Jiřetín",
        "Horní Planá",
        "Horní Slavkov",
        "Horšovský Týn",
        "Hostinné",
        "Hostivice",
        "Hostomice",
        "Hostouň",
        "Hořice",
        "Hořovice",
        "Hoštka",
        "Hradec Králové",
        "Hradec nad Moravicí",
        "Hranice (okres Cheb)",
        "Hrob",
        "Hrochův Týnec",
        "Hronov",
        "Hrotovice",
        "Hroznětín",
        "Hrušovany nad Jevišovkou",
        "Hrádek",
        "Hrádek nad Nisou",
        "Hulín",
        "Husinec",
        "Hustopeče",
        "Ivanovice na Hané",
        "Ivančice",
        "Jablonec nad Jizerou",
        "Jablonec nad Nisou",
        "Jablonné nad Orlicí",
        "Jablonné v Podještědí",
        "Jablunkov",
        "Janov",
        "Janovice nad Úhlavou",
        "Janské Lázně",
        "Jaroměřice nad Rokytnou",
        "Javorník",
        "Jemnice",
        "Jesenice (okres Rakovník)",
        "Jevišovice",
        "Jevíčko",
        "Jihlava",
        "Jilemnice",
        "Jistebnice",
        "Jiříkov",
        "Jáchymov",
        "Jílové",
        "Jílové u Prahy",
        "Kamenice nad Lipou",
        "Kamenický Šenov",
        "Kaplice",
        "Kardašova Řečice",
        "Karlovy Vary",
        "Karolinka",
        "Karviná",
        "Kasejovice",
        "Kaznějov",
        "Kašperské Hory",
        "Kdyně",
        "Kelč",
        "Kladno",
        "Kladruby",
        "Klecany",
        "Klimkovice",
        "Klobouky u Brna",
        "Kojetín",
        "Konice",
        "Kopidlno",
        "Koryčany",
        "Kosmonosy",
        "Kostelec na Hané",
        "Kostelec nad Labem",
        "Kostelec nad Orlicí",
        "Kostelec nad Černými lesy",
        "Kouřim",
        "Košťany",
        "Kožlany",
        "Kralovice",
        "Kraslice",
        "Kravaře",
        "Kryry",
        "Králíky",
        "Králův Dvůr",
        "Krásno",
        "Krásná Hora nad Vltavou",
        "Krásná Lípa",
        "Krásné Údolí",
        "Kunovice",
        "Kunštát",
        "Kynšperk nad Ohří",
        "Lanžhot",
        "Ledeč nad Sázavou",
        "Ledvice",
        "Letohrad",
        "Letovice",
        "Liberec",
        "Libochovice",
        "Libušín",
        "Libáň",
        "Libčice nad Vltavou",
        "Liběchov",
        "Lipník nad Bečvou",
        "Litovel",
        "Lišov",
        "Loket",
        "Lom",
        "Lomnice nad Lužnicí",
        "Lomnice nad Popelkou",
        "Loučná pod Klínovcem",
        "Lovosice",
        "Loštice",
        "Luby",
        "Luhačovice",
        "Lučany nad Nisou",
        "Luže",
        "Lysá nad Labem",
        "Lázně Bohdaneč",
        "Lázně Bělohrad",
        "Lázně Kynžvart",
        "Manětín",
        "Mašťov",
        "Meziboří",
        "Meziměstí",
        "Mikulov",
        "Mikulášovice",
        "Miletín",
        "Milevsko",
        "Milovice",
        "Mimoň",
        "Miroslav",
        "Mirotice",
        "Mirovice",
        "Mirošov",
        "Mladá Boleslav",
        "Mladá Vožice",
        "Mnichovice",
        "Mnichovo Hradiště",
        "Mníšek pod Brdy",
        "Modřice",
        "Mohelnice",
        "Moravské Budějovice",
        "Moravský Beroun",
        "Moravský Krumlov",
        "Morkovice-Slížany",
        "Most",
        "Mýto",
        "Městec Králové",
        "Město Albrechtice",
        "Město Touškov",
        "Měčín",
        "Mšeno",
        "Nalžovské Hory",
        "Napajedla",
        "Nasavrky",
        "Nechanice",
        "Nejdek",
        "Nepomuk",
        "Netolice",
        "Neveklov",
        "Nová Bystřice",
        "Nová Paka",
        "Nová Role",
        "Nová Včelnice",
        "Nové Hrady",
        "Nové Město nad Metují",
        "Nové Město pod Smrkem",
        "Nové Sedlo",
        "Nové Strašecí",
        "Nový Bydžov",
        "Nový Knín",
        "Náměšť nad Oslavou",
        "Nýrsko",
        "Nýřany",
        "Němčice nad Hanou",
        "Odolena Voda",
        "Odry",
        "Olešnice",
        "Olomouc",
        "Oloví",
        "Opava",
        "Opočno",
        "Osek",
        "Osečná",
        "Oslavany",
        "Ostrava",
        "Pacov",
        "Pardubice",
        "Paskov",
        "Pec pod Sněžkou",
        "Petřvald",
        "Pečky",
        "Pilníkov",
        "Planá",
        "Planá nad Lužnicí",
        "Plasy",
        "Plesná",
        "Plumlov",
        "Plzeň",
        "Plánice",
        "Poběžovice",
        "Podbořany",
        "Podivín",
        "Pohořelice",
        "Police nad Metují",
        "Polička",
        "Polná",
        "Postoloprty",
        "Potštát",
        "Počátky",
        "Praha",
        "Proseč",
        "Prostějov",
        "Protivín",
        "Pyšely",
        "Přebuz",
        "Přelouč",
        "Přerov",
        "Přeštice",
        "Přibyslav",
        "Přimda",
        "Příbor",
        "Rabí",
        "Radnice",
        "Rajhrad",
        "Ralsko",
        "Raspenava",
        "Rejštejn",
        "Rokytnice nad Jizerou",
        "Rokytnice v Orlických horách",
        "Ronov nad Doubravou",
        "Rosice",
        "Rotava",
        "Rousínov",
        "Rovensko pod Troskami",
        "Roztoky",
        "Rožmberk nad Vltavou",
        "Rožmitál pod Třemšínem",
        "Rožďalovice",
        "Rtyně v Podkrkonoší",
        "Rudná",
        "Rudolfov",
        "Rychnov u Jablonce nad Nisou",
        "Rychvald",
        "Rájec-Jestřebí",
        "Rýmařov",
        "Sadská",
        "Sedlec-Prčice",
        "Sedlice",
        "Sedlčany",
        "Semily",
        "Sezemice",
        "Sezimovo Ústí",
        "Seč",
        "Skalná",
        "Skuteč",
        "Slatiňany",
        "Slavičín",
        "Slavkov u Brna",
        "Slavonice",
        "Slušovice",
        "Smečno",
        "Smiřice",
        "Smržovka",
        "Sobotka",
        "Soběslav",
        "Solnice",
        "Spálené Poříčí",
        "Staré Město (okres Uherské Hradiště)",
        "Staré Město (okres Šumperk)",
        "Starý Plzenec",
        "Staňkov",
        "Stochov",
        "Stod",
        "Strmilov",
        "Stráž nad Nežárkou",
        "Stráž pod Ralskem",
        "Strážnice",
        "Strážov",
        "Studénka",
        "Stárkov",
        "Stříbro",
        "Suchdol nad Lužnicí",
        "Svoboda nad Úpou",
        "Svratka",
        "Světlá nad Sázavou",
        "Sázava",
        "Tanvald",
        "Telč",
        "Teplice",
        "Teplice nad Metují",
        "Teplá",
        "Terezín",
        "Tišnov",
        "Toužim",
        "Tovačov",
        "Trhové Sviny",
        "Trhový Štěpánov",
        "Trmice",
        "Týn nad Vltavou",
        "Týnec nad Labem",
        "Týnec nad Sázavou",
        "Týniště nad Orlicí",
        "Třebechovice pod Orebem",
        "Třebenice",
        "Třeboň",
        "Třemošnice",
        "Třemošná",
        "Třešť",
        "Uherský Ostroh",
        "Uhlířské Janovice",
        "Unhošť",
        "Valašské Klobouky",
        "Valtice",
        "Vamberk",
        "Vejprty",
        "Velešín",
        "Velká Bystřice",
        "Velká Bíteš",
        "Velké Bílovice",
        "Velké Hamry",
        "Velké Opatovice",
        "Velké Pavlovice",
        "Velký Šenov",
        "Veltrusy",
        "Velvary",
        "Verneřice",
        "Veselí nad Lužnicí",
        "Vidnava",
        "Vimperk",
        "Vizovice",
        "Vlachovo Březí",
        "Vodňany",
        "Volary",
        "Volyně",
        "Votice",
        "Vracov",
        "Vratimov",
        "Vrbno pod Pradědem",
        "Vroutek",
        "Vysoké Veselí",
        "Vysoké nad Jizerou",
        "Vyšší Brod",
        "Vítkov",
        "Výsluní",
        "Všeruby",
        "Zbiroh",
        "Zbýšov",
        "Zdice",
        "Zlaté Hory",
        "Zliv",
        "Zlín",
        "Zruč nad Sázavou",
        "Zubří",
        "Zákupy",
        "Zásmuky",
        "Újezd u Brna",
        "Úpice",
        "Úsov",
        "Ústí nad Labem",
        "Úterý",
        "Úvaly",
        "Úštěk",
        "Černovice",
        "Černošice",
        "Černošín",
        "Červená Řečice",
        "Červený Kostelec",
        "Česká Kamenice",
        "Česká Skalice",
        "České Budějovice",
        "České Velenice",
        "Český Brod",
        "Český Dub",
        "Řevnice",
        "Šenov",
        "Šlapanice",
        "Šluknov",
        "Špindlerův Mlýn",
        "Štramberk",
        "Štíty",
        "Štětí",
        "Švihov",
        "Žacléř",
        "Žamberk",
        "Žandov",
        "Ždánice",
        "Ždírec nad Doubravou",
        "Žebrák",
        "Železnice",
        "Železná Ruda",
        "Železný Brod",
        "Židlochovice",
        "Žirovnice",
        "Žlutice",
        "Žulová",
    )

    streets = (
        "Horní Stromky",
        "Vizovická",
        "K Brusce",
        "Mírová",
        "Rašínská",
        "Boušova",
        "Pobřežní",
        "Dolnobřežanská",
        "Černá",
        "Šůrova",
        "Červenkova",
        "Nad Mostem",
        "Libuňská",
        "Chotovická",
        "Petříkova",
        "Pod Vodárenskou Věží",
        "Na Fišerce",
        "Ke Březině",
        "Za Lázeňkou",
        "Nad Šafránkou",
        "Na Laurové",
        "Nám. Republiky",
        "Vlašimská",
        "Nad Rohatci",
        "Tylišovská",
        "Nábřeží Kapitána Jaroše",
        "Lešovská",
        "U Podjezdu",
        "Průškova",
        "Estonská",
        "Máslova",
        "K Otočce",
        "Jižní",
        "Švecova",
        "Mongolská",
        "Kalská",
        "Nad Rokytkou",
        "Malešovská",
        "Plzeňská",
        "V Hájkách",
        "Úpská",
        "Ambrožova",
        "Pikovická",
        "Neužilova",
        "Na Staré Vinici",
        "Vstupní",
        "Nýdecká",
        "U Společenské Zahrady",
        "Ostrovského",
        "Bazovského",
        "Lešenská",
        "Na Štamberku",
        "Na Svahu",
        "Výhledské Nám.",
        "K Lipám",
        "Za Stadionem",
        "Opletalova",
        "Nábřeží Ludvíka Svobody",
        "Komenského Nám.",
        "Křimická",
        "Domkovská",
        "Pyšelská",
        "Štychova",
        "Horákova",
        "Nad Zavážkou",
        "K Prelátům",
        "Vašátkova",
        "Benákova",
        "Náměstí Prezidenta Masaryka",
        "Mílovská",
        "U Hostivařského Nádraží",
        "Jihovýchodní I",
        "Hostivařské Nám.",
        "Zbynická",
        "Heineho",
        "U Dobešky",
        "Doubická",
        "Ke Břvům",
        "Na Záhonech",
        "Kloboukova",
        "Kostnické Náměstí",
        "Pelclova",
        "Smotlachova",
        "Pod Spiritkou",
        "Hůlkova",
        "Matenská",
        "Do Zahrádek Ii",
        "Dobrošovská",
        "Lovčenská",
        "Jasná I",
        "Škrétova",
        "Moravanů",
        "Budapešťská",
        "Kojetická",
        "Náměstí I. P. Pavlova",
        "Bajkalská",
        "U Větrolamu",
        "Vlčická",
        "Jarešova",
        "Sámova",
        "Kotrčová",
        "Musílkova",
        "Ingrišova",
        "U Nových Domů I",
        "Dělostřelecká",
        "Ke Hrázi",
        "Mochovská",
        "Rýmařovská",
        "Dolní Chaloupky",
        "Za Arielem",
        "U Rajské Zahrady",
        "K Šedivce",
        "Březová",
        "Doubravínova",
        "Mládkova",
        "Tachovské Náměstí",
        "Lehárova",
        "Severní X",
        "V Tehovičkách",
        "Bermanova",
        "Grammova",
        "Spojovací",
        "Verdunská",
        "Závrchy",
        "Čerpadlová",
        "Vítězná",
        "Nad Plynovodem",
        "U Smíchovského Hřbitova",
        "Nedvědovo Náměstí",
        "Bachova",
        "U Dálnice",
        "Všejanská",
        "Maňákova",
        "Rokytnická",
        "Loděnická",
        "U Pumpy",
        "Michnova",
        "Záblatská",
        "Poslední",
        "Hněvkovského",
        "Za Křížem",
        "Nad Návsí",
        "Jablonecká",
        "Súdánská",
        "Mazancova",
        "Pod Čertovou Skalou",
        "Weilova",
        "Čajkovského",
        "Nad Zátiším",
        "Moldavská",
        "Juarézova",
        "Žižkova",
        "Pod Lochkovem",
        "Nad Vernerákem",
        "Žherská",
        "Prusíkova",
        "Výtoňská",
        "Na Srážku",
        "Šachovská",
        "Nučická",
        "Novákovo Náměstí",
        "Sitteho",
        "U Vápenice",
        "Na Kuthence",
        "Čelakovského Sady",
        "V Závitu",
        "Na Vartě",
        "Oválová",
        "Machovická",
        "Nad Olšinami",
        "Vajgarská",
        "Kulhavého",
        "Kodaňská",
        "Kralupská",
        "Lednická",
        "Pod Velkým Hájem",
        "Hvězdonická",
        "Na Kozinci",
        "Semická",
        "K Dálnici",
        "Trytova",
        "Vyhlídkova",
        "Pohnertova",
        "U Nového Dvora",
        "K Vodě",
        "Nad Libří",
        "K Matěji",
        "V Kotcích",
        "Kohoutových",
        "Na Cikánce",
        "Chládkova",
        "Slatiňanská",
        "Pod Kostelem",
        "Na Spojce",
        "Na Zahrádkách",
        "Nad Obcí",
        "K Přehradám",
        "Na Náspu",
        "V Nížinách",
        "Josefa Houdka",
        "Na Pěšině",
        "Hnězdenská",
        "Za Statky",
        "Kremnická",
        "Čestmírova",
        "U Rakovky",
        "Kodicilova",
        "K Lučinám",
        "Nouzov",
        "Krátký Lán",
        "Anny Drabíkové",
        "Kadaňská",
        "Stroupežnického",
        "Jírova",
        "U Dětského Hřiště",
        "Žofie Podlipské",
        "Nad Šancemi",
        "Lošáková",
        "Roblínská",
        "Mezi Sklady",
        "Na Pomezí",
        "U Mlýnského Rybníka",
        "Makedonská",
        "K Dýmači",
        "V Zátiší",
        "Pohořelec",
        "Jiřinková",
        "U Nové Dálnice",
        "Čuprova",
        "Vraňanská",
        "Severovýchodní Vi",
        "Petřínská",
        "K Hořavce",
        "Sádovská",
        "Pod Průsekem",
        "Konžská",
        "Dřítenská",
        "Pirinská",
        "U Hřiště",
        "Kukelská",
        "Moravanská",
        "Koclířova",
        "Žilinská",
        "Ve Žlíbku",
        "Veronské Nám.",
        "U Větrníku",
        "Svojsíkova",
        "Izraelská",
        "Staňkovka",
        "Na Viničních Horách",
        "Čankovská",
        "Na Špitálce",
        "Valdovská",
        "Rudoltická",
        "Ke Strašnické",
        "Paťanka",
        "Panuškova",
        "Pankrácké Nám.",
        "Budčická",
        "Šermířská",
        "Medlovská",
        "K Vidouli",
        "Horní Chaloupky",
        "V Americe",
        "Dejvická",
        "Klášterecká",
        "Šárovo Kolo",
        "Mladoboleslavská",
        "Palackého",
        "Lumiérů",
        "Ivančická",
        "Za Valem",
        "Na Břevnovské Pláni",
        "Tichonická",
        "Náměstí Hrdinů",
        "Mistřínská",
        "Křížkovského",
        "Tanvaldská",
        "V Padolině",
        "Před Skalkami Ii",
        "Na Křivce",
        "Nad Zámečkem",
        "Nad Krocínkou",
        "Podlešínská",
        "Nad Popelkou",
        "Oderská",
        "Jeruzalémská",
        "Smolenská",
        "Lebeděvova",
        "Libichovská",
        "Na Šafránce",
        "Průjezdná",
        "Záluské",
        "Branišovská",
        "Spinozova",
        "K Betáni",
        "Machuldova",
        "Podohradská",
        "Cerhenická",
        "V Brůdku",
        "U Vlachovky",
        "Pod Letištěm",
        "Vlastislavova",
        "Klecanská",
        "Žinkovská",
        "Maltézské Náměstí",
        "Boršov",
        "Mukařovského",
        "Josefa Šimůnka",
        "Suchdolská",
        "Opočínská",
        "Heydukova",
        "Vršovka",
        "Thurnova",
        "Mezilesní",
        "Za Pivovarem",
        "Uljanovská",
        "Panenská",
        "Sladovnická",
        "Plynární",
        "Kozácká",
        "Vlasákova",
        "Javornická",
        "Ševčíkova",
        "Podle Náhonu",
        "Doubravická",
        "Františka Černého",
        "Chotětovská",
        "K Háječku",
        "Pod Výšinkou",
        "U Šesté Baterie",
        "Drahanská",
        "Augustova",
        "U Balabenky",
        "Boční I",
        "Jirčanská",
        "Na Šubě",
        "Brixiho",
        "Klímova",
        "Kazín",
        "Fügnerovo Náměstí",
        "Na Příčné Mezi",
        "Plánická",
        "Africká",
        "Vratislavova",
        "Olympijská",
        "Na Bojišti",
        "K Nádrži",
        "Vokrojova",
        "Bořetínská",
        "Kováříkova",
        "Lánovská",
        "U Staré Pošty",
        "Na Poustkách",
        "V Poli",
        "Meziškolská",
        "Pajerova",
        "Habartovská",
        "Mlékárenská",
        "Dělnická",
        "U Štěpu",
        "Družná",
        "Klouzková",
        "Před Rybníkem",
        "Nad Košinkou",
        "Spolupráce",
        "V Humenci",
        "Adélčina",
        "Březanova",
        "Pod Kesnerkou",
        "Kosmonoská",
        "Do Dubin",
        "Nad Lávkou",
        "Mezi Lysinami",
        "Na Topolce",
        "Snopkova",
        "Severní Viii",
        "Okrová",
        "Třebihošťská",
        "Mádrova",
        "Na Lázeňce",
        "Slivenecká",
        "Nám. Barikád",
        "Nad Strouhou",
        "Jindřicha Plachty",
        "Pod Srázem",
        "U Waltrovky",
        "Bratří Čapků",
        "Onšovecká",
        "Machnova",
        "Kostková",
        "Rožmberská",
        "Zapských",
        "Přípřežní",
        "Výravská",
        "Podléšková",
        "Štěchovická",
        "Poleradská",
        "Jilmová",
        "Hostýnská",
    )

    states = (
        "Hlavní město Praha",
        "Středočeský kraj",
        "Jihočeský kraj",
        "Plzeňský kraj",
        "Karlovarský kraj",
        "Ústecký kraj",
        "Liberecký kraj",
        "Královéhradecký kraj",
        "Pardubický kraj",
        "Kraj Vysočina",
        "Jihomoravský kraj",
        "Olomoucký kraj",
        "Moravskoslezský kraj",
        "Zlínský kraj",
    )

    countries = (
        "Afghánistán",
        "Albánie",
        "Alžírsko",
        "Andorra",
        "Angola",
        "Antigua a Barbuda",
        "Argentina",
        "Arménie",
        "Austrálie",
        "Bahamy",
        "Bahrajn",
        "Bangladéš",
        "Barbados",
        "Belgie",
        "Belize",
        "Benin",
        "Bhútán",
        "Bolívie",
        "Bosna a Hercegovina",
        "Botswana",
        "Brazílie",
        "Brunej",
        "Bulharsko",
        "Burkina Faso",
        "Burundi",
        "Bělorusko",
        "Chile",
        "Chorvatsko",
        "Cookovy ostrovy",
        "Demokratická republika Kongo",
        "Dominika",
        "Dominikánská republika",
        "Dánsko",
        "Džibutsko",
        "Egypt",
        "Ekvádor",
        "Eritrea",
        "Estonsko",
        "Etiopie",
        "Federativní státy Mikronésie",
        "Fidži",
        "Filipíny",
        "Finsko",
        "Francie",
        "Gabon",
        "Gambie",
        "Ghana",
        "Gruzie",
        "Guatemala",
        "Guinea",
        "Guinea-Bissau",
        "Guyana",
        "Haiti",
        "Honduras",
        "Indie",
        "Irsko",
        "Irák",
        "Island",
        "Itálie",
        "Izrael",
        "Jamajka",
        "Japonsko",
        "Jemen",
        "Jihoafrická republika",
        "Jižní Súdán",
        "Jordánsko",
        "Kambodža",
        "Kamerun",
        "Kanada",
        "Kapverdy",
        "Katar",
        "Kazachstán",
        "Keňa",
        "Kiribati",
        "Kolumbie",
        "Kostarika",
        "Kuba",
        "Kypr",
        "Kyrgyzstán",
        "Laos",
        "Lesotho",
        "Libanon",
        "Libye",
        "Lichtenštejnsko",
        "Litva",
        "Lotyšsko",
        "Lucembursko",
        "Madagaskar",
        "Makedonie",
        "Malajsie",
        "Malawi",
        "Maledivy",
        "Mali",
        "Malta",
        "Maroko",
        "Marshallovy ostrovy",
        "Mauricius",
        "Mauritánie",
        "Maďarsko",
        "Mexiko",
        "Moldavsko",
        "Monako",
        "Mongolsko",
        "Mosambik",
        "Myanmar",
        "Namibie",
        "Nauru",
        "Nepál",
        "Niger",
        "Nigérie",
        "Nikaragua",
        "Niue",
        "Nizozemsko",
        "Norsko",
        "Nový Zéland",
        "Německo",
        "Omán",
        "Palau",
        "Panama",
        "Papua-Nová Guinea",
        "Paraguay",
        "Peru",
        "Pobřeží slonoviny",
        "Polsko",
        "Portugalsko",
        "Pákistán",
        "Rakousko",
        "Republika Kongo",
        "Rovníková Guinea",
        "Rumunsko",
        "Rusko",
        "Rwanda",
        "Salvador",
        "Samoa",
        "San Marino",
        "Saúdská Arábie",
        "Senegal",
        "Severní Korea",
        "Seychely",
        "Sierra Leone",
        "Singapur",
        "Slovensko",
        "Slovinsko",
        "Somálsko",
        "Spojené arabské emiráty",
        "Spojené království",
        "Spojené státy americké",
        "Srbsko",
        "Středoafrická republika",
        "Surinam",
        "Svatá Lucie",
        "Svatý Kryštof a Nevis",
        "Svatý Tomáš a Princův ostrov",
        "Svatý Vincenc a Grenadiny",
        "Svazijsko",
        "Súdán",
        "Sýrie",
        "Tanzanie",
        "Thajsko",
        "Togo",
        "Tonga",
        "Trinidad a Tobago",
        "Tunisko",
        "Turecko",
        "Turkmenistán",
        "Tuvalu",
        "Tádžikistán",
        "Uganda",
        "Ukrajina",
        "Uruguay",
        "Uzbekistán",
        "Vanuatu",
        "Vatikán",
        "Venezuela",
        "Vietnam",
        "Východní Timor",
        "Zambie",
        "Zimbabwe",
        "Ázerbájdžán",
        "Írán",
        "Čad",
        "Černá Hora",
        "Česko",
        "Čína",
        "Řecko",
        "Šalamounovy ostrovy",
        "Španělsko",
        "Srí Lanka",
        "Švédsko",
        "Švýcarsko",
    )

    def street_suffix_short(self) -> str:
        return self.random_element(self.street_suffixes_short)

    def street_suffix_long(self) -> str:
        return self.random_element(self.street_suffixes_long)

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def street_name(self) -> str:
        return self.random_element(self.streets)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit

    def city_with_postcode(self) -> str:
        return self.postcode() + " " + self.random_element(self.cities)



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    # Source:
    # https://en.wikipedia.org/wiki/Telephone_numbers_in_the_State_of_Palestine

    cellphone_formats = (
        "{{area_code}} {{provider_code}} ### ####",
        "{{area_code}}{{provider_code}}#######",
        "0{{provider_code}} ### ####",
        "0{{provider_code}}#######",
    )

    telephone_formats = (
        "{{area_code}} 4 24# ####",
        "{{area_code}}424#####",
        "04 24# ####",
        "0424#####",
        "{{area_code}} 9 25# ####",
        "{{area_code}}925#####",
        "09 25# ####",
        "0925#####",
        "{{area_code}} 4 26# ####",
        "{{area_code}}426#####",
        "04 26# ####",
        "0426#####",
        "{{area_code}} 4 23# ####",
        "{{area_code}}423#####",
        "04 23# ####",
        "0423#####",
        "{{area_code}} 4 29# ####",
        "{{area_code}}429#####",
        "04 29# ####",
        "0429#####",
        "{{area_code}} 2 29# ####",
        "{{area_code}}229#####",
        "02 29# ####",
        "0229#####",
        "{{area_code}} 2 23# ####",
        "{{area_code}}223#####",
        "02 23# ####",
        "0223#####",
        "{{area_code}} 2 22# ####",
        "{{area_code}}222#####",
        "02 22# ####",
        "0222#####",
        "{{area_code}} 2 27# ####",
        "{{area_code}}227#####",
        "02 27# ####",
        "0227#####",
        "{{area_code}} 8 20# ####",
        "{{area_code}}820#####",
        "08 20# ####",
        "0820#####",
        "{{area_code}} 8 21# ####",
        "{{area_code}}821#####",
        "08 21# ####",
        "0821#####",
        "{{area_code}} 8 24# ####",
        "{{area_code}}824#####",
        "08 24# ####",
        "0824#####",
        "{{area_code}} 8 25# ####",
        "{{area_code}}825#####",
        "08 25# ####",
        "0825#####",
        "{{area_code}} 8 26# ####",
        "{{area_code}}826#####",
        "08 26# ####",
        "0826#####",
        "{{area_code}} 8 28# ####",
        "{{area_code}}828#####",
        "08 28# ####",
        "0828#####",
    )

    toll_formats = (
        "1 700 ### ###",
        "1-700-###-###",
        "1 800 ### ###",
        "1-800-###-###",
    )

    services_phones_formats = (
        "100",
        "101",
        "102",
    )

    formats = cellphone_formats + telephone_formats + services_phones_formats + toll_formats

    def provider_code(self) -> str:
        return self.random_element(
            [
                "59",
                "56",
            ]
        )

    def area_code(self) -> str:
        return self.random_element(
            [
                "00972",
                "+972",
                "00970",
                "+970",
            ]
        )

    def cellphone_number(self) -> str:
        pattern: str = self.random_element(self.cellphone_formats)
        return self.numerify(self.generator.parse(pattern))

    def telephone_number(self) -> str:
        pattern: str = self.random_element(self.telephone_formats)
        return self.numerify(self.generator.parse(pattern))

    def service_phone_number(self) -> str:
        pattern: str = self.random_element(self.services_phones_formats)
        return self.numerify(self.generator.parse(pattern))

    def toll_number(self) -> str:
        pattern: str = self.random_element(self.toll_formats)
        return self.numerify(self.generator.parse(pattern))

    def phone_number(self) -> str:
        pattern: str = self.random_element(self.formats)
        return self.numerify(self.generator.parse(pattern))



from typing import List

from .. import Provider as CompanyProvider


def company_id_checksum(digits: List[int]) -> List[int]:
    digits = list(digits)
    weights = 6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2

    dv = sum(w * d for w, d in zip(weights[1:], digits))
    dv = (11 - dv) % 11
    dv = 0 if dv >= 10 else dv
    digits.append(dv)

    dv2 = sum(w * d for w, d in zip(weights, digits))
    dv2 = (11 - dv2) % 11
    dv2 = 0 if dv2 >= 10 else dv2
    digits.append(dv2)

    return digits[-2:]


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
        "{{last_name}}",
    )

    catch_phrase_formats = ("{{catch_phrase_noun}} {{catch_phrase_verb}} {{catch_phrase_attribute}}",)

    nouns = (
        "a segurança",
        "o prazer",
        "o conforto",
        "a simplicidade",
        "a certeza",
        "a arte",
        "o poder",
        "o direito",
        "a possibilidade",
        "a vantagem",
        "a liberdade",
    )

    verbs = (
        "de conseguir",
        "de avançar",
        "de evoluir",
        "de mudar",
        "de inovar",
        "de ganhar",
        "de atingir seus objetivos",
        "de concretizar seus projetos",
        "de realizar seus sonhos",
    )

    attributes = (
        "de maneira eficaz",
        "mais rapidamente",
        "mais facilmente",
        "simplesmente",
        "com toda a tranquilidade",
        "antes de tudo",
        "naturalmente",
        "sem preocupação",
        "em estado puro",
        "com força total",
        "direto da fonte",
        "com confiança",
    )

    company_suffixes = ("S/A", "S.A.", "Ltda.", "- ME", "- EI", "e Filhos")

    def catch_phrase_noun(self) -> str:
        """
        Returns a random catch phrase noun.
        """
        return self.random_element(self.nouns)

    def catch_phrase_attribute(self) -> str:
        """
        Returns a random catch phrase attribute.
        """
        return self.random_element(self.attributes)

    def catch_phrase_verb(self) -> str:
        """
        Returns a random catch phrase verb.
        """
        return self.random_element(self.verbs)

    def catch_phrase(self) -> str:
        """
        :example 'a segurança de evoluir sem preocupação'
        """
        pattern: str = self.random_element(self.catch_phrase_formats)
        catch_phrase = self.generator.parse(pattern)
        catch_phrase = catch_phrase[0].upper() + catch_phrase[1:]
        return catch_phrase

    def company_id(self) -> str:
        digits: List[int] = list(self.random_sample(range(10), 8))
        digits += [0, 0, 0, 1]
        digits += company_id_checksum(digits)
        return "".join(str(d) for d in digits)

    def cnpj(self) -> str:
        digits = self.company_id()
        return f"{digits[:2]}.{digits[2:5]}.{digits[5:8]}/{digits[8:12]}-{digits[12:]}"



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        "onet.pl",
        "interia.pl",
        "gmail.com",
        "o2.pl",
        "yahoo.com",
        "hotmail.com",
    )

    tlds = ("com", "com", "com", "net", "org", "pl", "pl", "pl")

    replacements = (
        ("ą", "a"),
        ("ć", "c"),
        ("ę", "e"),
        ("ł", "l"),
        ("ń", "n"),
        ("ó", "o"),
        ("ś", "s"),
        ("ź", "z"),
        ("ż", "z"),
    )



from collections import OrderedDict

from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = OrderedDict(
        (
            ("{{company_limited_prefix}}{{last_name}} {{company_limited_suffix}}", 0.2),
            (
                "{{company_limited_prefix}}{{last_name}}{{company_suffix}} {{company_limited_suffix}}",
                0.2,
            ),
            ("{{company_limited_prefix}}{{last_name}} {{company_limited_suffix}}", 0.2),
            ("{{company_prefix}}{{last_name}}", 0.2),
            ("{{company_prefix}}{{last_name}}{{company_suffix}}", 0.2),
            ("{{last_name}}{{company_suffix}}", 0.1),
            ("{{nonprofit_prefix}}{{last_name}}", 0.1),
            ("{{last_name}}-{{last_name}}", 0.05),
            ("{{last_name}}และ{{last_name}}", 0.05),
            ("{{company_limited_prefix}}{{last_name}}", 0.01),
        )
    )

    company_prefixes = OrderedDict(
        (
            ("ห้างหุ้นส่วนจำกัด ", 0.3),
            ("หจก.", 0.2),
            ("บจก.", 0.1),
            ("บมจ.", 0.1),
            ("ห้างหุ้นส่วนสามัญ ", 0.1),
            ("หสน.", 0.01),
        )
    )

    nonprofit_prefixes = OrderedDict(
        (
            ("สมาคม", 0.4),
            ("มูลนิธิ", 0.3),
            ("ชมรม", 0.2),
            ("สหภาพแรงงาน", 0.1),
        )
    )

    company_suffixes = (
        "และเพื่อน",
        "และบุตร",
        "แอนด์ซันส์",
        "กรุ๊ป",
        "การช่าง",
        "ก่อสร้าง",
        "บริการ",
        "เซอร์วิส",
        "กลการ",
        "ซัพพลาย",
        "คอมมิวนิเคชั่น",
        "พืชผล",
        "เอเยนซี",
        "เอ็นจิเนียริ่ง",
        "คอนสตรัคชั่น",
        "วิศวกรรม",
        "วิศวการ",
        "คอมพิวเตอร์",
        "พานิช",
        "ขนส่ง",
        "เฟอนิชชิ่ง",
        "เฟอร์นิเจอร์",
        "อุตสาหกรรม",
        "เอนเตอรไพรส์",
        "จิวเวลรี่",
        "อะไหล่ยนต์",
        "ภาพยนตร์",
        "ยานยนต์",
        "เทรดดิ้ง",
        "การค้า",
        "แลบ",
        "เคมิคอล",
        "อิมปอร์ตเอ็กซปอร์ต",
        "อินเตอร์เนชั่นแนล",
        "บรรจุภัณฑ์",
        "แพคกิ้ง",
        "มอเตอร์",
        "โอสถ",
        "การบัญชี",
        "สโตร์",
    )

    company_limited_prefixes = OrderedDict(
        (
            ("บริษัท ", 0.95),
            ("ธนาคาร", 0.03),
            ("บริษัทหลักทรัพย์ ", 0.005),
            ("กองทุนรวม", 0.005),
        )
    )

    company_limited_suffixes = OrderedDict(
        (
            ("จำกัด", 0.85),
            ("จำกัด (มหาชน)", 0.15),
        )
    )

    def company_prefix(self) -> str:
        """
        :example 'ห้างหุ้นส่วนจำกัด'
        """
        return self.random_element(self.company_prefixes)

    def company_limited_prefix(self) -> str:
        """
        :example 'บริษัท'
        """
        return self.random_element(self.company_limited_prefixes)

    def company_limited_suffix(self) -> str:
        """
        :example 'จำกัด'
        """
        return self.random_element(self.company_limited_suffixes)

    def nonprofit_prefix(self) -> str:
        """
        :example 'มูลนิธิ'
        """
        return self.random_element(self.nonprofit_prefixes)



from .. import Provider as ColorProvider


class Provider(ColorProvider):
    """Implement color provider for ``hu_HU`` locale."""

    safe_colors = (
        "fekete",
        "bordó",
        "zöld",
        "királykék",
        "oliva",
        "bíbor",
        "kékeszöld",
        "citromzöld",
        "kék",
        "ezüst",
        "szürke",
        "sárga",
        "mályva",
        "akvamarin",
        "fehér",
    )



from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``la`` locale."""

    word_list = (
        "a",
        "ab",
        "accusamus",
        "accusantium",
        "ad",
        "adipisci",
        "alias",
        "aliquam",
        "aliquid",
        "amet",
        "animi",
        "aperiam",
        "architecto",
        "asperiores",
        "aspernatur",
        "assumenda",
        "at",
        "atque",
        "aut",
        "autem",
        "beatae",
        "blanditiis",
        "commodi",
        "consectetur",
        "consequatur",
        "consequuntur",
        "corporis",
        "corrupti",
        "culpa",
        "cum",
        "cumque",
        "cupiditate",
        "debitis",
        "delectus",
        "deleniti",
        "deserunt",
        "dicta",
        "dignissimos",
        "distinctio",
        "dolor",
        "dolore",
        "dolorem",
        "doloremque",
        "dolores",
        "doloribus",
        "dolorum",
        "ducimus",
        "ea",
        "eaque",
        "earum",
        "eius",
        "eligendi",
        "enim",
        "eos",
        "error",
        "esse",
        "est",
        "et",
        "eum",
        "eveniet",
        "ex",
        "excepturi",
        "exercitationem",
        "expedita",
        "explicabo",
        "facere",
        "facilis",
        "fuga",
        "fugiat",
        "fugit",
        "harum",
        "hic",
        "id",
        "illo",
        "illum",
        "impedit",
        "in",
        "incidunt",
        "inventore",
        "ipsa",
        "ipsam",
        "ipsum",
        "iste",
        "itaque",
        "iure",
        "iusto",
        "labore",
        "laboriosam",
        "laborum",
        "laudantium",
        "libero",
        "magnam",
        "magni",
        "maiores",
        "maxime",
        "minima",
        "minus",
        "modi",
        "molestiae",
        "molestias",
        "mollitia",
        "nam",
        "natus",
        "necessitatibus",
        "nemo",
        "neque",
        "nesciunt",
        "nihil",
        "nisi",
        "nobis",
        "non",
        "nostrum",
        "nulla",
        "numquam",
        "occaecati",
        "odio",
        "odit",
        "officia",
        "officiis",
        "omnis",
        "optio",
        "pariatur",
        "perferendis",
        "perspiciatis",
        "placeat",
        "porro",
        "possimus",
        "praesentium",
        "provident",
        "quae",
        "quaerat",
        "quam",
        "quas",
        "quasi",
        "qui",
        "quia",
        "quibusdam",
        "quidem",
        "quis",
        "quisquam",
        "quo",
        "quod",
        "quos",
        "ratione",
        "recusandae",
        "reiciendis",
        "rem",
        "repellat",
        "repellendus",
        "reprehenderit",
        "repudiandae",
        "rerum",
        "saepe",
        "sapiente",
        "sed",
        "sequi",
        "similique",
        "sint",
        "sit",
        "soluta",
        "sunt",
        "suscipit",
        "tempora",
        "tempore",
        "temporibus",
        "tenetur",
        "totam",
        "ullam",
        "unde",
        "ut",
        "vel",
        "velit",
        "veniam",
        "veritatis",
        "vero",
        "vitae",
        "voluptas",
        "voluptate",
        "voluptatem",
        "voluptates",
        "voluptatibus",
        "voluptatum",
    )



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):

    # Source: http://www.localeplanet.com/icu/ta-IN/index.html
    DAY_NAMES = {
        "0": "திங்கள்",
        "1": "செவ்வாய்",
        "2": "புதன்",
        "3": "வியாழன்",
        "4": "வெள்ளி",
        "5": "சனி",
        "6": "ஞாயிறு",
    }

    MONTH_NAMES = {
        "01": "ஜனவரி",
        "02": "பிப்ரவரி",
        "03": "மார்ச்",
        "04": "ஏப்ரல்",
        "05": "மே",
        "06": "ஜூன்",
        "07": "ஜூலை",
        "08": "ஆகஸ்ட்",
        "09": "செப்டம்பர்",
        "10": "அக்டோபர்",
        "11": "நவம்பர்",
        "12": "டிசம்பர்",
    }

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``pt_PT`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Portugal
    """

    license_formats = (
        "##-##-??",
        "##-??-##",
        "??-##-##",
        # New format since March 2020
        "??-##-??",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{prefix}} {{first_name}} {{last_name}}",
    )

    prefixes_male = (
        "GEN",
        "LT GEN",
        "MAJ GEN",
        "COL",
        "LT COL",
        "MAJ",
        "CAPT",
        "LT",
        "SUB LT",
        "S M 1",
        "S M 2",
        "S M 3",
        "SGT",
        "CPL",
        "PFC",
        "PVT",
        "ADM",
        "V ADM",
        "R ADM",
        "CAPT",
        "CDR",
        "L CDR",
        "LT",
        "LT JG",
        "SUB LT",
        "CPO 1",
        "CPO 2",
        "CPO 3",
        "PO 1",
        "PO 2",
        "PO 3",
        "SEA-MAN",
        "ACM",
        "AM",
        "AVM",
        "GP CAPT",
        "WG CDR",
        "SQN LDR",
        "FLT LT",
        "FLG OFF",
        "PLT OFF",
        "FS 1",
        "FS 2",
        "FS 3",
        "SGT",
        "CPL",
        "LAC",
        "AMN",
        "POL GEN",
        "POL LT GEN",
        "POL MAJ GEN",
        "POL COL",
        "POL LT COL",
        "POL MAJ",
        "POL CAPT",
        "POL LT",
        "POL SUB LT",
        "POL SEN SGT MAJ",
        "POL SGT MAJ",
        "POL SGT",
        "POL CPL",
        "POL L/C",
        "POL CONST",
        "MR",
        "REV",
        "M L",
        "M R",
        "SAMANERA",
        "PHRA",
        "PHRA ATHIKAN",
        "CHAO ATHIKAN",
        "PHRAPALAD",
        "PHRASAMU",
        "PHRABAIDIKA",
        "PHRAKHU PALAD",
        "PHRAKHU SAMU",
        "PHRAKHU BAIDIKA",
        "PHRAMAHA",
        "PHRAKHU DHAMMADHORN",
        "PHRAKHU VINAIDHORN",
    )

    prefixes_female = (
        "GEN",
        "LT GEN",
        "MAJ GEN",
        "COL",
        "LT COL",
        "MAJ",
        "CAPT",
        "LT",
        "SUB LT",
        "S M 1",
        "S M 2",
        "S M 3",
        "SGT",
        "CPL",
        "PFC",
        "PVT",
        "ADM",
        "V ADM",
        "R ADM",
        "CAPT",
        "CDR",
        "L CDR",
        "LT",
        "LT JG",
        "SUB LT",
        "CPO 1",
        "CPO 2",
        "CPO 3",
        "PO 1",
        "PO 2",
        "PO 3",
        "SEA-MAN",
        "ACM",
        "AM",
        "AVM",
        "GP CAPT",
        "WG CDR",
        "SQN LDR",
        "FLT LT",
        "FLG OFF",
        "PLT OFF",
        "FS 1",
        "FS 2",
        "FS 3",
        "SGT",
        "CPL",
        "LAC",
        "AMN",
        "POL GEN",
        "POL LT GEN",
        "POL MAJ GEN",
        "POL COL",
        "POL LT COL",
        "POL MAJ",
        "POL CAPT",
        "POL LT",
        "POL SUB LT",
        "POL SEN SGT MAJ",
        "POL SGT MAJ",
        "POL SGT",
        "POL CPL",
        "POL L/C",
        "POL CONST",
        "MRS",
        "MISS",
        "REV",
        "M L",
    )

    prefixes = prefixes_male + prefixes_female

    first_names = (
        "Pornchanok",
        "Patchaploy",
        "Peem",
        "Kodchaporn",
        "Pattapon",
        "Sarunporn",
        "Jinjuta",
        "Sorawut",
        "Suvakit",
        "Prima",
        "Darin",
        "Pintusorn",
        "Kulnun",
        "Nutcha",
        "Nutkrita",
        "Sittikorn",
        "Wasin",
        "Apisara",
        "Nattawun",
        "Tunradee",
        "Niracha",
        "Tunchanok",
        "Kamolchanok",
        "Jaruwan",
        "Pachongruk",
        "Pakjira",
        "Pattatomporn",
        "Suwijuk",
        "Noppakao",
        "Ratchanon",
        "Atit",
        "Kunaporn",
        "Arisara",
        "Todsawun",
        "Chaiwut",
        "Puntira",
        "Supasita",
        "Patcharaporn",
        "Phubes",
        "Pattamon",
        "Chanya",
        "Pannawich",
        "Chawin",
        "Pada",
        "Chanikan",
        "Nutwadee",
        "Chalisa",
        "Prames",
        "Supasit",
        "Sitiwat",
        "Teetat",
        "Yada",
        "Phenphitcha",
        "Anon",
        "Chaifah",
        "Pawan",
        "Aunyaporn",
        "Yanisa",
        "Pak",
        "Chayanin",
        "Chayapat",
        "Jitrin",
        "Wassaya",
        "Pitipat",
        "Nichakarn",
        "Parin",
        "Thanatcha",
    )

    last_names = (
        "Prachayaroch",
        "Prachayaroch",
        "Kamalanon",
        "Tianvarich",
        "Bunlerngsri",
        "Sukhenai",
        "Posalee",
        "Chaisatit",
        "Sujjaboriboon",
        "Kamalanon",
        "Neerachapong",
        "Pianduangsri",
        "Pasuk",
        "Losatapornpipit",
        "Suraprasert",
        "Matinawin",
        "Choeychuen",
        "Wasunun",
        "Kumsoontorn",
        "Sireelert",
        "Boonpungbaramee",
        "Sorattanachai",
        "Benchapatranon",
        "Intaum",
        "Pikatsingkorn",
        "Srisoontorn",
        "Polpo",
        "Kongchayasukawut",
        "Charoensuksopol",
        "Bunlupong",
        "Chomsri",
        "Tungkasethakul",
        "Chowitunkit",
        "Todsapornpitakul",
        "Wimolnot",
        "Kittakun",
        "Methavorakul",
        "Pitanuwat",
        "Phusilarungrueng",
        "Turongkinanon",
        "Kitprapa",
        "Pothanun",
        "Youprasert",
        "Methavorakul",
        "Vethayasas",
        "Sooksawang",
        "Anekvorakul",
        "Pichpandecha",
        "Sittisaowapak",
        "Suraprachit",
        "Kongsri",
        "Trikasemmart",
        "Habpanom",
        "Wannapaitoonsri",
        "Vinyuvanichkul",
        "Pongpanitch",
        "Permchart",
        "Chaihirankarn",
        "Thantananont",
        "Norramon",
        "Prayoonhong",
        "Lertsattayanusak",
        "Polauaypon",
        "Prakalpawong",
        "Titipatrayunyong",
        "Krittayanukoon",
        "Siripaiboo",
    )



import csv
import hashlib
import io
import json
import re
import string
import tarfile
import uuid
import zipfile

from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, Union

from faker.exceptions import UnsupportedFeature

from .. import BaseProvider

localized = True

csv.register_dialect("faker-csv", csv.excel, quoting=csv.QUOTE_ALL)


class Provider(BaseProvider):
    def boolean(self, chance_of_getting_true: int = 50) -> bool:
        """Generate a random boolean value based on ``chance_of_getting_true``.

        :sample size=10: chance_of_getting_true=25
        :sample size=10: chance_of_getting_true=50
        :sample size=10: chance_of_getting_true=75
        """
        return self.generator.random.randint(1, 100) <= chance_of_getting_true

    def null_boolean(self) -> Optional[bool]:
        """Generate ``None``, ``True``, or ``False``, each with equal probability.

        :sample size=15:
        """
        return {
            0: None,
            1: True,
            -1: False,
        }[self.generator.random.randint(-1, 1)]

    def binary(self, length: int = (1 * 1024 * 1024)) -> bytes:
        """Generate a random binary blob of ``length`` bytes.

        :sample: length=64
        """
        blob = [self.generator.random.randrange(256) for _ in range(length)]
        return bytes(blob)

    def md5(self, raw_output: bool = False) -> Union[bytes, str]:
        """Generate a random MD5 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the MD5 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res: hashlib._Hash = hashlib.md5(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha1(self, raw_output: bool = False) -> Union[bytes, str]:
        """Generate a random SHA1 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA1 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res: hashlib._Hash = hashlib.sha1(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha256(self, raw_output: bool = False) -> Union[bytes, str]:
        """Generate a random SHA256 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA56 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res: hashlib._Hash = hashlib.sha256(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def uuid4(
        self,
        cast_to: Optional[Union[Callable[[uuid.UUID], str], Callable[[uuid.UUID], bytes]]] = str,
    ) -> Union[bytes, str, uuid.UUID]:
        """Generate a random UUID4 object and cast it to another type if specified using a callable ``cast_to``.

        By default, ``cast_to`` is set to ``str``.

        May be called with ``cast_to=None`` to return a full-fledged ``UUID``.

        :sample:
        :sample: cast_to=None
        """
        # Based on http://stackoverflow.com/q/41186818
        generated_uuid: uuid.UUID = uuid.UUID(int=self.generator.random.getrandbits(128), version=4)
        if cast_to is not None:
            return cast_to(generated_uuid)
        return generated_uuid

    def password(
        self,
        length: int = 10,
        special_chars: bool = True,
        digits: bool = True,
        upper_case: bool = True,
        lower_case: bool = True,
    ) -> str:
        """Generate a random password of the specified ``length``.

        The arguments ``special_chars``, ``digits``, ``upper_case``, and ``lower_case`` control
        what category of characters will appear in the generated password. If set to ``True``
        (default), at least one character from the corresponding category is guaranteed to appear.
        Special characters are characters from ``!@#$%^&*()_+``, digits are characters from
        ``0123456789``, and uppercase and lowercase characters are characters from the ASCII set of
        letters.

        :sample: length=12
        :sample: length=40, special_chars=False, upper_case=False
        """
        choices = ""
        required_tokens = []
        if special_chars:
            required_tokens.append(self.generator.random.choice("!@#$%^&*()_+"))
            choices += "!@#$%^&*()_+"
        if digits:
            required_tokens.append(self.generator.random.choice(string.digits))
            choices += string.digits
        if upper_case:
            required_tokens.append(self.generator.random.choice(string.ascii_uppercase))
            choices += string.ascii_uppercase
        if lower_case:
            required_tokens.append(self.generator.random.choice(string.ascii_lowercase))
            choices += string.ascii_lowercase

        assert len(required_tokens) <= length, "Required length is shorter than required characters"

        # Generate a first version of the password
        chars: str = self.random_choices(choices, length=length)  # type: ignore

        # Pick some unique locations
        random_indexes: Set[int] = set()
        while len(random_indexes) < len(required_tokens):
            random_indexes.add(self.generator.random.randint(0, len(chars) - 1))

        # Replace them with the required characters
        for i, index in enumerate(random_indexes):
            chars[index] = required_tokens[i]  # type: ignore

        return "".join(chars)

    def zip(
        self,
        uncompressed_size: int = 65536,
        num_files: int = 1,
        min_file_size: int = 4096,
        compression: Optional[str] = None,
    ) -> bytes:
        """Generate a bytes object containing a random valid zip archive file.

        The number and sizes of files contained inside the resulting archive can be controlled
        using the following arguments:

        - ``uncompressed_size`` - the total size of files before compression, 16 KiB by default
        - ``num_files`` - the number of files archived in resulting zip file, 1 by default
        - ``min_file_size`` - the minimum size of each file before compression, 4 KiB by default

        No compression is used by default, but setting ``compression`` to one of the values listed
        below will use the corresponding compression type.

        - ``'bzip2'`` or ``'bz2'`` for BZIP2
        - ``'lzma'`` or ``'xz'`` for LZMA
        - ``'deflate'``, ``'gzip'``, or ``'gz'`` for GZIP

        :sample: uncompressed_size=256, num_files=4, min_file_size=32
        :sample: uncompressed_size=256, num_files=32, min_file_size=4, compression='bz2'
        """
        if any(
            [
                not isinstance(num_files, int) or num_files <= 0,
                not isinstance(min_file_size, int) or min_file_size <= 0,
                not isinstance(uncompressed_size, int) or uncompressed_size <= 0,
            ]
        ):
            raise ValueError(
                "`num_files`, `min_file_size`, and `uncompressed_size` must be positive integers",
            )
        if min_file_size * num_files > uncompressed_size:
            raise AssertionError(
                "`uncompressed_size` is smaller than the calculated minimum required size",
            )
        if compression in ["bzip2", "bz2"]:
            compression_ = zipfile.ZIP_BZIP2
        elif compression in ["lzma", "xz"]:
            compression_ = zipfile.ZIP_LZMA
        elif compression in ["deflate", "gzip", "gz"]:
            compression_ = zipfile.ZIP_DEFLATED
        else:
            compression_ = zipfile.ZIP_STORED

        zip_buffer = io.BytesIO()
        remaining_size = uncompressed_size
        with zipfile.ZipFile(zip_buffer, mode="w", compression=compression_) as zip_handle:
            for file_number in range(1, num_files + 1):
                filename = self.generator.pystr() + str(file_number)

                max_allowed_size = remaining_size - (num_files - file_number) * min_file_size
                if file_number < num_files:
                    file_size = self.generator.random.randint(min_file_size, max_allowed_size)
                    remaining_size = remaining_size - file_size
                else:
                    file_size = remaining_size

                data = self.generator.binary(file_size)
                zip_handle.writestr(filename, data)
        return zip_buffer.getvalue()

    def tar(
        self,
        uncompressed_size: int = 65536,
        num_files: int = 1,
        min_file_size: int = 4096,
        compression: Optional[str] = None,
    ) -> bytes:
        """Generate a bytes object containing a random valid tar file.

        The number and sizes of files contained inside the resulting archive can be controlled
        using the following arguments:

        - ``uncompressed_size`` - the total size of files before compression, 16 KiB by default
        - ``num_files`` - the number of files archived in resulting zip file, 1 by default
        - ``min_file_size`` - the minimum size of each file before compression, 4 KiB by default

        No compression is used by default, but setting ``compression`` to one of the values listed
        below will use the corresponding compression type.

        - ``'bzip2'`` or ``'bz2'`` for BZIP2
        - ``'lzma'`` or ``'xz'`` for LZMA
        - ``'gzip'`` or ``'gz'`` for GZIP

        :sample: uncompressed_size=256, num_files=4, min_file_size=32
        :sample: uncompressed_size=256, num_files=32, min_file_size=4, compression='bz2'
        """
        if any(
            [
                not isinstance(num_files, int) or num_files <= 0,
                not isinstance(min_file_size, int) or min_file_size <= 0,
                not isinstance(uncompressed_size, int) or uncompressed_size <= 0,
            ]
        ):
            raise ValueError(
                "`num_files`, `min_file_size`, and `uncompressed_size` must be positive integers",
            )
        if min_file_size * num_files > uncompressed_size:
            raise AssertionError(
                "`uncompressed_size` is smaller than the calculated minimum required size",
            )
        if compression in ["gzip", "gz"]:
            mode = "w:gz"
        elif compression in ["bzip2", "bz2"]:
            mode = "w:bz2"
        elif compression in ["lzma", "xz"]:
            mode = "w:xz"
        else:
            mode = "w"

        tar_buffer = io.BytesIO()
        remaining_size = uncompressed_size
        with tarfile.open(mode=mode, fileobj=tar_buffer) as tar_handle:
            for file_number in range(1, num_files + 1):
                file_buffer = io.BytesIO()
                filename = self.generator.pystr() + str(file_number)

                max_allowed_size = remaining_size - (num_files - file_number) * min_file_size
                if file_number < num_files:
                    file_size = self.generator.random.randint(min_file_size, max_allowed_size)
                    remaining_size = remaining_size - file_size
                else:
                    file_size = remaining_size

                tarinfo = tarfile.TarInfo(name=filename)
                data = self.generator.binary(file_size)
                file_buffer.write(data)
                tarinfo.size = len(file_buffer.getvalue())
                file_buffer.seek(0)
                tar_handle.addfile(tarinfo, file_buffer)
                file_buffer.close()
        return tar_buffer.getvalue()

    def image(
        self,
        size: Tuple[int, int] = (256, 256),
        image_format: str = "png",
        hue: Optional[Union[int, Sequence[int], str]] = None,
        luminosity: Optional[str] = None,
    ) -> bytes:
        """Generate an image and draw a random polygon on it using the Python Image Library.
        Without it installed, this provider won't be functional. Returns the bytes representing
        the image in a given format.

        The argument ``size`` must be a 2-tuple containing (width, height) in pixels. Defaults to 256x256.

        The argument ``image_format`` can be any valid format to the underlying library like ``'tiff'``,
        ``'jpeg'``, ``'pdf'`` or ``'png'`` (default). Note that some formats need present system libraries
        prior to building the Python Image Library.
        Refer to https://pillow.readthedocs.io/en/stable/handbook/image-file-formats.html for details.

        The arguments ``hue`` and ``luminosity`` are the same as in the color provider and are simply forwarded to
        it to generate both the background and the shape colors. Therefore, you can ask for a "dark blue" image, etc.

        :sample size=2: size=(2, 2), hue='purple', luminosity='bright', image_format='pdf'
        :sample size=2: size=(16, 16), hue=[90,270], image_format='ico'
        """
        try:
            import PIL.Image
            import PIL.ImageDraw
        except ImportError:
            raise UnsupportedFeature("`image` requires the `Pillow` python library.", "image")

        (width, height) = size
        image = PIL.Image.new("RGB", size, self.generator.color(hue=hue, luminosity=luminosity))
        draw = PIL.ImageDraw.Draw(image)
        draw.polygon(
            [(self.random_int(0, width), self.random_int(0, height)) for _ in range(self.random_int(3, 12))],
            fill=self.generator.color(hue=hue, luminosity=luminosity),
            outline=self.generator.color(hue=hue, luminosity=luminosity),
        )
        with io.BytesIO() as fobj:
            image.save(fobj, format=image_format)
            fobj.seek(0)
            return fobj.read()

    def dsv(
        self,
        dialect: str = "faker-csv",
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
        **fmtparams: Any,
    ) -> str:
        """Generate random delimiter-separated values.

        This method's behavior share some similarities with ``csv.writer``. The ``dialect`` and
        ``**fmtparams`` arguments are the same arguments expected by ``csv.writer`` to control its
        behavior, and instead of expecting a file-like object to where output will be written, the
        output is controlled by additional keyword arguments and is returned as a string.

        The ``dialect`` argument defaults to ``'faker-csv'`` which is the name of a ``csv.excel``
        subclass with full quoting enabled.

        The ``header`` argument expects a list or a tuple of strings that will serve as the header row
        if supplied. The ``data_columns`` argument expects a list or a tuple of string tokens, and these
        string tokens will be passed to  :meth:`pystr_format() <faker.providers.python.Provider.pystr_format>`
        for data generation. Argument Groups are used to pass arguments to the provider methods.
        Both ``header`` and ``data_columns`` must be of the same length.

        Example:
            fake.set_arguments('top_half', {'min_value': 50, 'max_value': 100})
            fake.dsv(data_columns=('{{ name }}', '{{ pyint:top_half }}'))

        The ``num_rows`` argument controls how many rows of data to generate, and the ``include_row_ids``
        argument may be set to ``True`` to include a sequential row ID column.

        :sample: dialect='excel', data_columns=('{{name}}', '{{address}}')
        :sample: dialect='excel-tab', data_columns=('{{name}}', '{{address}}'), include_row_ids=True
        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=5, delimiter='$'
        """

        if not isinstance(num_rows, int) or num_rows <= 0:
            raise ValueError("`num_rows` must be a positive integer")
        if not isinstance(data_columns, (list, tuple)):
            raise TypeError("`data_columns` must be a tuple or a list")
        if header is not None:
            if not isinstance(header, (list, tuple)):
                raise TypeError("`header` must be a tuple or a list")
            if len(header) != len(data_columns):
                raise ValueError("`header` and `data_columns` must have matching lengths")

        dsv_buffer = io.StringIO()
        writer = csv.writer(dsv_buffer, dialect=dialect, **fmtparams)

        if header:
            if include_row_ids:
                header = list(header)
                header.insert(0, "ID")
            writer.writerow(header)

        for row_num in range(1, num_rows + 1):
            row = [self.generator.pystr_format(column) for column in data_columns]
            if include_row_ids:
                row.insert(0, str(row_num))

            writer.writerow(row)

        return dsv_buffer.getvalue()

    def csv(
        self,
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
    ) -> str:
        """Generate random comma-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header,
            data_columns=data_columns,
            num_rows=num_rows,
            include_row_ids=include_row_ids,
            delimiter=",",
        )

    def tsv(
        self,
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
    ) -> str:
        """Generate random tab-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header,
            data_columns=data_columns,
            num_rows=num_rows,
            include_row_ids=include_row_ids,
            delimiter="\t",
        )

    def psv(
        self,
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
    ) -> str:
        """Generate random pipe-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header,
            data_columns=data_columns,
            num_rows=num_rows,
            include_row_ids=include_row_ids,
            delimiter="|",
        )

    def json(self, data_columns: List = None, num_rows: int = 10, indent: int = None) -> str:
        """
        Generate random JSON structure values.

        Using a dictionary or list of records that is passed as ``data_columns``,
        define the structure that is used to build JSON structures.  For complex
        data structures it is recommended to use the dictionary format.

        Data Column Dictionary format:
            {'key name': 'definition'}

        The definition can be 'provider', 'provider:argument_group', tokenized
        'string {{ provider:argument_group }}' that is passed to the python
        provider method pystr_format() for generation, or a fixed '@word'.
        Using Lists, Tuples, and Dicts as a definition for structure.

        Example:
            fake.set_arguments('top_half', {'min_value': 50, 'max_value': 100})
            fake.json(data_columns={'Name': 'name', 'Score': 'pyint:top_half'})

        Data Column List format:
            [('key name', 'definition', {'arguments'})]

        With the list format the definition can be a list of records, to create
        a list within the structure data.  For literal entries within the list,
        set the 'field_name' to None.

        :param data_columns: specification for the data structure
        :type data_columns: dict
        :param num_rows: number of rows the returned
        :type num_rows: int
        :param indent: number of spaces to indent the fields
        :type indent: int
        :return: Serialized JSON data
        :rtype: str

        :sample: data_columns={'Spec': '@1.0.1', 'ID': 'pyint',
                'Details': {'Name': 'name', 'Address': 'address'}}, num_rows=2
        :sample: data_columns={'Candidates': ['name', 'name', 'name']},
                num_rows=1
        :sample: data_columns=[('Name', 'name'), ('Points', 'pyint',
                {'min_value': 50, 'max_value': 100})], num_rows=1
        """
        default_data_columns = {
            "name": "{{name}}",
            "residency": "{{address}}",
        }
        data_columns: Union[List, Dict] = data_columns if data_columns else default_data_columns

        def process_list_structure(data: Sequence[Any]) -> Any:
            entry: Dict[str, Any] = {}

            for name, definition, *arguments in data:
                kwargs = arguments[0] if arguments else {}

                if not isinstance(kwargs, dict):
                    raise TypeError("Invalid arguments type. Must be a dictionary")

                if name is None:
                    return self._value_format_selection(definition, **kwargs)

                if isinstance(definition, tuple):
                    entry[name] = process_list_structure(definition)
                elif isinstance(definition, (list, set)):
                    entry[name] = [process_list_structure([item]) for item in definition]
                else:
                    entry[name] = self._value_format_selection(definition, **kwargs)
            return entry

        def process_dict_structure(data: Union[int, float, bool, Dict[str, Any]]) -> Any:
            entry: Dict[str, Any] = {}

            if isinstance(data, str):
                return self._value_format_selection(data)

            if isinstance(data, dict):
                for name, definition in data.items():
                    if isinstance(definition, (tuple, list, set)):
                        entry[name] = [process_dict_structure(item) for item in definition]
                    elif isinstance(definition, (dict, int, float, bool)):
                        entry[name] = process_dict_structure(definition)
                    else:
                        entry[name] = self._value_format_selection(definition)
                return entry

            return data

        def create_json_structure(data_columns: Union[Dict, List]) -> dict:
            if isinstance(data_columns, dict):
                return process_dict_structure(data_columns)

            if isinstance(data_columns, list):
                return process_list_structure(data_columns)

            raise TypeError("Invalid data_columns type. Must be a dictionary or list")

        if num_rows == 1:
            return json.dumps(create_json_structure(data_columns), indent=indent)

        data = [create_json_structure(data_columns) for _ in range(num_rows)]
        return json.dumps(data, indent=indent)

    def fixed_width(self, data_columns: list = None, num_rows: int = 10, align: str = "left") -> str:
        """
        Generate random fixed width values.

        Using a list of tuple records that is passed as ``data_columns``, that
        defines the structure that will be generated. Arguments within the
        record are provider specific, and should be a dictionary that will be
        passed to the provider method.

        Data Column List format
            [('field width', 'definition', {'arguments'})]

        The definition can be 'provider', 'provider:argument_group', tokenized
        'string {{ provider:argument_group }}' that is passed to the python
        provider method pystr_format() for generation, or a fixed '@word'.
        Using Lists, Tuples, and Dicts as a definition for structure.

        Argument Groups can be used to pass arguments to the provider methods,
        but will override the arguments supplied in the tuple record.

        Example:
            fake.set_arguments('top_half', {'min_value': 50, 'max_value': 100})
            fake.fixed_width(data_columns=[(20, 'name'), (3, 'pyint:top_half')])

        :param data_columns: specification for the data structure
        :type data_columns: list
        :param num_rows: number of rows the generator will yield
        :type num_rows: int
        :param align: positioning of the value. (left, middle, right)
        :type align: str
        :return: Serialized Fixed Width data
        :rtype: str

        :sample: data_columns=[(20, 'name'), (3, 'pyint', {'min_value': 50,
                'max_value': 100})], align='right', num_rows=2
        """
        default_data_columns = [
            (20, "name"),
            (3, "pyint", {"max_value": 20}),
        ]
        data_columns = data_columns if data_columns else default_data_columns
        align_map = {
            "left": "<",
            "middle": "^",
            "right": ">",
        }
        data = []

        for _ in range(num_rows):
            row = []

            for width, definition, *arguments in data_columns:
                kwargs = arguments[0] if arguments else {}

                if not isinstance(kwargs, dict):
                    raise TypeError("Invalid arguments type. Must be a dictionary")

                result = self._value_format_selection(definition, **kwargs)
                row.append(f'{result:{align_map.get(align, "<")}{width}}'[:width])

            data.append("".join(row))
        return "\n".join(data)

    def _value_format_selection(self, definition: str, **kwargs: Any) -> Union[int, str]:
        """
        Formats the string in different ways depending on it's contents.

        The return can be the '@word' itself, a '{{ token }}' passed to PyStr,
        or a 'provider:argument_group' format field that returns potentially
        a non-string type.

        This ensures that Numbers, Boolean types that are generated in the
        JSON structures in there proper type, and not just strings.
        """

        # Check for PyStr first as complex strings may start with @
        if re.match(r".*\{\{.*\}\}.*", definition):
            return self.generator.pystr_format(definition)

        # Check for fixed @words that won't be generated
        if re.match(r"^@.*", definition):
            return definition.lstrip("@")

        # Check if a argument group has been supplied
        if re.match(r"^[a-zA-Z0-9_-]*:\w", definition):
            definition, argument_group = definition.split(":")
            arguments = self.generator.get_arguments(argument_group.strip())

            return self.generator.format(definition.strip(), **arguments)

        # Assume the string is refering to a provider
        return self.generator.format(definition, **kwargs)



from collections import OrderedDict

from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    """
    Provider for company names for en_PH locale

    Company naming scheme and probabilities are inspired by and/or based on existing companies in the Philippines.

    Sources:
    - https://en.wikipedia.org/wiki/List_of_companies_of_the_Philippines
    - https://www.pse.com.ph/stockMarket/listedCompanyDirectory.html
    """

    formats = OrderedDict(
        [
            (
                "{{random_company_adjective}} {{random_company_noun_chain}} {{company_type}} {{company_suffix}}",
                0.24,
            ),
            (
                "{{random_company_acronym}} {{random_company_noun_chain}} {{company_type}} {{company_suffix}}",
                0.24,
            ),
            (
                "{{last_name}} {{random_company_noun_chain}} {{company_type}} {{company_suffix}}",
                0.16,
            ),
            ("{{random_company_adjective}} {{company_type}} {{company_suffix}}", 0.12),
            ("{{random_company_acronym}} {{company_type}} {{company_suffix}}", 0.12),
            ("{{last_name}} {{company_type}} {{company_suffix}}", 0.09),
            (
                "National {{random_company_product}} Corporation of the Philippines",
                0.03,
            ),
        ]
    )
    company_suffixes = OrderedDict(
        [
            ("Inc.", 0.45),
            ("Corporation", 0.45),
            ("Limited", 0.1),
        ]
    )
    company_types = (
        "Bank",
        "Banking",
        "Capital",
        "Company",
        "Construction",
        "Development",
        "Enterprise",
        "Equities",
        "Finance",
        "Foods",
        "Group",
        "Holdings",
        "Hotel",
        "Manufacturing",
        "Mining",
        "Properties",
        "Resorts",
        "Resources",
        "Services",
        "Shipping",
        "Solutions",
        "Technologies",
        "Trust",
        "Ventures",
    )
    company_products = (
        "Bottle",
        "Coconut",
        "Computer",
        "Electricity",
        "Flour",
        "Furniture",
        "Glass",
        "Newspaper",
        "Pillow",
        "Water",
    )
    company_nouns = (
        "Century",
        "City",
        "Crown",
        "Dragon",
        "Empire",
        "Genesis",
        "Gold",
        "King",
        "Liberty",
        "Millennium",
        "Morning",
        "Silver",
        "Star",
        "State",
        "Summit",
        "Sun",
        "Union",
        "World",
    )
    company_adjectives = (
        "Advanced",
        "Rising",
        "Double",
        "Triple",
        "Quad",
        "Allied",
        "Cyber",
        "Sovereign",
        "Great",
        "Far",
        "Northern",
        "Southern",
        "Eastern",
        "Western",
        "First",
        "Filipino",
        "Grand",
        "Manila",
        "Mega",
        "Metro",
        "Global",
        "Pacific",
        "Oriental",
        "Philippine",
        "Prime",
    )

    def company_type(self) -> str:
        return self.random_element(self.company_types)

    def random_company_adjective(self) -> str:
        return self.random_element(self.company_adjectives)

    def random_company_noun_chain(self) -> str:
        return " ".join(self.random_elements(self.company_nouns, length=self.random_int(1, 2), unique=True))

    def random_company_product(self) -> str:
        return self.random_element(self.company_products)

    def random_company_acronym(self) -> str:
        letters = self.random_letters(self.random_int(2, 4))
        return "".join(letters).upper()
